package org.gaugebook;

import java.awt.BasicStroke;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Stroke;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.image.VolatileImage;

public class AbstractDirectionalGyroGauge extends AbstractClockGauge {

	protected VolatileImage directionalGyroVI;
	protected Graphics2D directionalGyroScaleGraphics;

	private static final long serialVersionUID = 1L;
	    
	public AbstractDirectionalGyroGauge(GaugeBook owner, String title) {
		super();
	}

	protected void drawDirectionalGyroScaleGraphics(final double r, final Point2D cp) {
        double r0 = proz(r,65);
        double r1 = proz(r,80);
        
        double majorTickSize = proz(r,7);
        double minorTickSize = proz(r,5);
        
        double majorTickR =  r1 + 5; // major tick radius 
        double minorTickR =  majorTickR + (majorTickSize - minorTickSize)/2.0; // minor tick radius

		GraphicsConfiguration gc = getGraphicsConfiguration();
		directionalGyroVI = gc.createCompatibleVolatileImage( getWidth(), getHeight() );
        
        do {
        	
        	gc = this.getGraphicsConfiguration();
        	if( this.directionalGyroVI.validate(gc) == VolatileImage.IMAGE_INCOMPATIBLE ) {
        		directionalGyroVI = gc.createCompatibleVolatileImage( getWidth(), getHeight() );
        	}
        	
        	directionalGyroScaleGraphics = this.directionalGyroVI.createGraphics();
        	
	        Stroke oldStroke = directionalGyroScaleGraphics.getStroke();
	        Font   oldFont   = directionalGyroScaleGraphics.getFont();
	
	        // draw minor ticks
	        if(minorTickStep > 0){
	        	directionalGyroScaleGraphics.setColor( minorTickColor );
	        	directionalGyroScaleGraphics.setStroke( new BasicStroke(1.f) );            
	            double minorGradSteps = angleExtend * minorTickStep / (maxValue - minValue);
	            for(double i= angleStart + angleExtend; i >= angleStart ; i-=minorGradSteps){
	                Point2D p0 = getPoint( cp, minorTickR, i );
	                Point2D p1 = getPoint( cp, minorTickR+minorTickSize, i );
	                directionalGyroScaleGraphics.draw( new Line2D.Double(p0,p1) );
	            }
	        }
	        
	        // draw major ticks and markers
	        if(majorTickStep > 0){
	            
	            double markerR = majorTickR + 2 * majorTickSize;
	            double l = valueToString( maxValue ).length();
	            if(l > 3 && decPlaceCount > 0){
	                l--;
	            }
	            double f = l < 4 ? 1.0 : 1.0 - 0.1 * (l - 3);   // font factor  
	            double majorFontSize = Math.max( 6.0,proz(r,12.0) * f);  // minimum 6.0
	            Font font = oldFont.deriveFont(majorFontSize < 8.0 ? Font.PLAIN : Font.BOLD, (float)majorFontSize );
	            directionalGyroScaleGraphics.setFont( font);        
	            directionalGyroScaleGraphics.setColor( majorTickColor );
	            directionalGyroScaleGraphics.setStroke( new BasicStroke(2.f) );            
	            
	            double stepAngle = angleExtend * majorTickStep/(maxValue-minValue);
	            for( double v = minValue; v<= maxValue; v+=majorTickStep ) {
	                double currentAngle = 90; // angleStart + angleExtend - stepAngle * (v - minValue) / majorTickStep;
	                Point2D p0 = getPoint( cp, majorTickR, currentAngle );
	                Point2D p1 = getPoint( cp, majorTickR + majorTickSize, currentAngle );
	                directionalGyroScaleGraphics.draw( new Line2D.Double( p0, p1 ) );
	                
	                // draw markers
	                String s;
	                switch( (int)Math.round( v / 10 )) {
	                case  0: s = "N"; break;
	                case  9: s = "E"; break;
	                case 18: s = "S"; break;
	                case 27: s = "W"; break;
	                default:
	                	 s = valueToString( v / 10 );
	                }
	                Dimension size = getStringSize( s, directionalGyroScaleGraphics );
	                double delta = size.getWidth() / 2;
	                double fontR = markerR - 3 * majorTickSize;
	                Point2D mc = getPoint( cp, fontR - delta, currentAngle ); // marker center point
	                directionalGyroScaleGraphics.drawString( s, (float)(mc.getX() - size.getWidth() / 2), (float)(mc.getY() + size.getHeight() / 2) );
	                directionalGyroScaleGraphics.rotate( Math.PI * majorTickStep / 180, cp.getX(), cp.getY() );
	            }
	            directionalGyroScaleGraphics.setStroke( oldStroke );
	            directionalGyroScaleGraphics.setFont( oldFont );         
	        }	
        } while( this.directionalGyroVI.contentsLost() );
    }
}
