package org.gaugebook;

import java.util.HashMap;
import java.util.Vector;

import flightsim.simconnect.SimConnectDataType;

/**
 * FSXVariables
 * 
 * see Simconnect SDK for Description
 *  
 * @author Michael Nagler
 *
 */
public class FSXVariable extends FSXVariablesHashMap {

	public static int INDEX_NAME = 0;
	public static int INDEX_DESC = 1;
	public static int INDEX_UNIT = 2;
	public static int INDEX_SET = 3;
	
	private String name;
	private String description;
	private String unit;
	private boolean settable;
	private Object value = null;
	private SimConnectDataType type;
	
	private static int varcount = 0;
	
	public FSXVariable( String name, String description, String unit, String settable ) {
		this.setName(name);
		this.setDescription(description);
		this.unit = unit;
		this.settable = settable.equals("true");

		System.out.print("FSXVar "+ (varcount++) +" '"+name.toUpperCase()+"' created, datatype: ");

		if( unit.equals("")) {
			this.type = SimConnectDataType.STRING32;
			System.out.print("STRING32");
		}
		else if( unit.equals("String8")) {
			this.type = SimConnectDataType.STRING8;
			System.out.print("STRING8");
		}
		else if( unit.equals("String") || unit.equals("String32")) {
			this.type = SimConnectDataType.STRING32;
			System.out.print("STRING32");
		}
		else if( unit.equals("String64")) {
			this.type = SimConnectDataType.STRING64;
			System.out.print("STRING64");
		}
		else if( unit.equals("STRING128")) {
			this.type = SimConnectDataType.STRING128;
			System.out.print("STRING128");
		}
		else if( unit.equals("String256")) {
			this.type = SimConnectDataType.STRING256;
			System.out.print("STRING256");
		}
		else if( unit.equals("Integer")) {
			this.type = SimConnectDataType.INT32;
			System.out.print("INT32");
		}
		else {
			this.type = SimConnectDataType.FLOAT64;
			System.out.print("FLOAT64");
		}
		System.out.println(" unit: "+ unit );
	}

	public static FSXVariable getVar( String name ) {
		String verify = name.toLowerCase();

		if( verify.indexOf(":") > 0 ) {
			verify = verify.substring(0,name.indexOf(":")) + ":index";
		}
		
		if( FSXVariablesHashMap.fsxVars.containsKey( verify )) {
			String[] var = fsxVars.get( verify );
			return new FSXVariable( name.toUpperCase(), var[1], var[2], var[3]);
		}
		System.out.println("FSXVar "+ name +" not found.");
		System.exit(1);
		return null;
	}

	private void setName( String name ) {
		this.name = name;
	}
	
	public String getName() {
		return name;
	}

	private void setDescription( String desc ) {
		this.description = desc;
	}

	public String getDescription() {
		return description;
	}
	
	public String getUnit() {
		return unit;
	}
	
	public boolean isSettable() {
		return settable;
	}

	protected Object getValue() {
		return value;
	}

	protected void setValue(Object value) {
		this.value = value;
	}

	/**
	 * Returns this FSX Variable as double.
	 * @return This FSX Variable as double
	 */
	public synchronized double getDoubleValue() {
		return value == null ? 0 : (Double)value;
	}

	/**
	 * Returns the value of this fsx var as boolean (jsimconnect seems to return sometimes 
	 * -1 and sometimes 1 for true, so I just use Math.abs here - just hope it's always 0 when false)
	 * @return This FSX Variable as boolean.
	 */
	public boolean getBooleanValue() {
		return value == null ? false : Math.abs((Double)value) == 1.0;
	}
	
	public String getStringValue() {
		return value == null ? "" :(String)value;
	}
	
	public void setType(SimConnectDataType type) {
		this.type = type;
	}

	public SimConnectDataType getType() {
		return type;
	}
	
	public String toString() {
		return this.name;
	}
	
	public static void reset() {
		varcount = 0;
	}
}
