/* GaugeHook - network interface to multiple Sims for GaugeBook
 * 
 * Copyright (C) 2009 Michael Nagler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.gaugebook;

// import simconnect classes

// import fsuipc classes
import com.flightsim.fsuipc.FSADF;
import com.flightsim.fsuipc.FSAircraft;
import com.flightsim.fsuipc.FSEngine1;
import com.flightsim.fsuipc.FSEngine2;
import com.flightsim.fsuipc.FSFlightSim;
import com.flightsim.fsuipc.FSGear;
import com.flightsim.fsuipc.FSNav1;
import com.flightsim.fsuipc.fsuipc_wrapper;
import com.flightsim.fsuipc.*;

/**
 * GaugeHook is a tool, that hooks into the target sim and provides it's data for GaugeBook.
 * It's used as interface to connect to FSX, FS2004, X-Plane and other Sims using SimConnect,
 * FSUICP or other interfaces.
 * 
 * @author Michael Nagler
 */
public class GaugeHook 
{
	static final String appName = "GaugeHook";
	static final String appVersion = "0.1 alpha";
	
	static final int FSX_SIMCONNECT = 1;
	static final int FSX_FSUIPC = 2;
	static final int FS9_FSUIPC = 3;
	static final int FS8_FSUIPC = 4;
	static final int XPLANE = 5;
	static final int FLIGHTGEAR = 6;
	
	Thread listener = null;
	
	public GaugeHook( String ip, int port, int target ) {
		int ret;
		switch( target ) {
		case FSX_SIMCONNECT:
			// connect to simconnect
			
			// ...
			
			break;
		case FSX_FSUIPC:
			// connect to fsuipc
			
			break;
		case FS9_FSUIPC:
			// connect to fsuipc
			ret = fsuipc_wrapper.Open(fsuipc_wrapper.SIM_ANY);
			if(ret == 0 )
			{
				System.out.println("Error: no flight sim found (return value = "+ ret +")!");
			}
			else
			{
				System.out.println("Connected to Flight Simulator 2004 (hopefully)!");
				// launch listen thread
				// ...
			}
			
			break;
		case FS8_FSUIPC:
			// connect to fsuipc
			ret = fsuipc_wrapper.Open(fsuipc_wrapper.SIM_FS2K2 );
			if(ret == 0 )
			{
				System.out.println("Error: no flight sim found (return value = "+ ret +")!");
			}
			else
			{
				System.out.println("Connected to MS Flight Simulator 2002!");
				// launch listen thread
				// ...
			}
			
			break;
		case XPLANE:
			// connect to xplane
			
			break;
		case FLIGHTGEAR:
			// connect to flightgear
			
			break;
		default:
			
		}
	}
}