package org.gaugebook;

import java.lang.reflect.Constructor;
import java.util.Vector;

/**
 * GaugeManager
 * 
 * The GaugeManager dynamically loads Gauges (deriving AbstractGauge).
 *  
 * @author Ants At War Team
 */
public class GaugeManager extends Object
{
	// instance variables
	protected Vector<AbstractGauge> gauges = new Vector<AbstractGauge>();
	
	// constructor
	public GaugeManager()
	{
		super();
	}
	
	// class methods
	
	/**
	 * 
	 */
	public static Object load(String strategy)
	{
		try
		{
			// load the named class
			Class cl = Class.forName(strategy);
			
			// in order to use newInstance() there has to be a no-argument constructor
			boolean hasNoArgumentConstructor = false;
			
			// get all constructors of the loaded class
			Constructor[] constructors = cl.getConstructors();
			for (int i = 0; i < constructors.length; i++)
			{
				// test its arguments
				if (constructors[i].getParameterTypes().length == 0)
				{
					hasNoArgumentConstructor = true;
					break;
				}
			}
			
			// if the class has no no-argument constructor it is not a valid
			// Strategy class
			if (!hasNoArgumentConstructor)
				return null;
			
			// instantiate a new object
			Object obj = cl.newInstance(); 
			
			return obj;
		}
		catch (Exception e) { return null; }
	}		
	
	public static boolean validateStrategy(String strategy)
	{
		return (load(strategy) instanceof AbstractGauge);
	}
	
	public boolean loadGauge(String gauge)
	{
		Object obj = load(gauge);
		if (obj instanceof AbstractGauge)
		{
			
			AbstractGauge s = (AbstractGauge)obj;
			gauges.add(s);
			
			//Simulator.information("StrategyManager: " + strategy + " " + Language.getString(Language.STRATEGYMANAGER_VERBOSE_SUCCLOAD));
			return true;
		}
		else
		{
			//Simulator.information("StrategyManager: " + strategy + " " + Language.getString(Language.STRATEGYMANAGER_VERBOSE_FAILLOAD));
			return false;
		}
	}
	
	public void clear()
	{
		gauges.removeAllElements();
		// Simulator.information("StrategyManager: " + Language.getString(Language.STRATEGYMANAGER_VERBOSE_CLEAR));
	}
	
/*	public String getInfo(int strategy)
	{
		return gauges.elementAt(strategy).getInfo();
	}
*/	
	public Object getProperty(AbstractGauge gauge, String property )
	{
		return gauge.getProperty(property);
	}

	public void setProperty(AbstractGauge gauge, String property, Object value)
	{
		gauge.setProperty(property, value);
	}

	public String toString()
	{
		return "(GaugeManager: " + gauges.size() + " strategies loaded)";
	}
}