package org.gaugebook;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

import org.gaugebook.swing.JDecimalTextField;
import org.gaugebook.swing.JFrequencyTextField;
import org.gaugebook.swing.JTransponderTextField;


public class InputDialog extends JDialog implements ActionListener, KeyListener {

	JTextField textField = null;
	private GaugeBook owner;
	private String event;

	public enum Unit {
		INT, BCDXPDR, BCD16, BCD16COMFREQ, BCD16NAVFREQ, BCD16HZ, BCD32, KOHLSMANINHG, DEGREES
	}
	
	private final Unit unit;
	
	public InputDialog( GaugeBook owner, String title, String event, Unit unit ) {
		super( owner, title, true );
		setSize(400, 100);
		this.owner = owner;
		this.event = event;
		this.unit  = unit;
		
		JPanel mp = new JPanel( new BorderLayout() ); // main panel
		JPanel jp = new JPanel( new FlowLayout() );  // input panel
		JPanel bp = new JPanel( new GridLayout( 4, 3 ) );  // button panel
		
		if( unit == Unit.BCD16COMFREQ || unit == Unit.BCD16NAVFREQ || unit == Unit.BCD16HZ) {
			textField = new JFrequencyTextField(6);
		} else if( unit == Unit.BCDXPDR ) {
			textField = new JTransponderTextField(4);
		} else {
			textField = new JDecimalTextField(6);
		}
		textField.setSize(100,20);
		textField.setMinimumSize( new Dimension( 100, 20));

	    textField.addFocusListener(new FocusAdapter() {
	        public void focusGained(final FocusEvent fe) {
	          SwingUtilities.invokeLater(new Runnable() {
	            public void run() {
	              ((JTextField)fe.getSource()).selectAll();}});}});
	    
	    addKeyListener(this);
	    textField.addKeyListener(this);
	    
		JButton btnOK = new JButton("OK");
		btnOK.addActionListener(this);
		
		jp.add( new JLabel(title));
		jp.add( textField );
		jp.add( btnOK );
		
		mp.add( jp, BorderLayout.NORTH );
		
		JButton b1 = new JButton("1");
		JButton b2 = new JButton("2");
		JButton b3 = new JButton("3");
		JButton b4 = new JButton("4");
		JButton b5 = new JButton("5");
		JButton b6 = new JButton("6");
		JButton b7 = new JButton("7");
		JButton b8 = new JButton("8");
		JButton b9 = new JButton("9");
		JButton b0 = new JButton("0");
		JButton bc = new JButton(".");
		
		b1.addActionListener( this );
		b2.addActionListener( this );
		b3.addActionListener( this );
		b4.addActionListener( this );
		b5.addActionListener( this );
		b6.addActionListener( this );
		b7.addActionListener( this );
		b8.addActionListener( this );
		b9.addActionListener( this );
		b0.addActionListener( this );
		bc.addActionListener( this );
		
		bp.add( b1 );
		bp.add( b2 );
		bp.add( b3 );
		bp.add( b4 );
		bp.add( b5 );
		bp.add( b6 );
		bp.add( b7 );
		bp.add( b8 );
		bp.add( b9 );
		bp.add( b0 );
		bp.add( bc );
		
		mp.add( bp, BorderLayout.CENTER );
		
        // set location on screen
        Dimension size = owner.getSize();
        setLocation((int)owner.getLocationOnScreen().getX() + ( size.width - getSize().width ) / 2 ,
        			(int)owner.getLocationOnScreen().getY() + ( size.height - getSize().height ) / 2 );

		add(mp);
		pack();
	}
	
	public void connect() {
		this.setVisible(false);
	}
	
	public String getText() {
		return this.textField.getText();
	}
	
	public void setText(String text) {
		textField.setText(text);
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		JButton s = new JButton("OK");

		if( e.getSource().getClass().toString().equals( "class javax.swing.JButton" )) {
			s = (JButton)e.getSource();
		}
		if( s.getText() == "1" ) { this.setText( this.getText().concat("1") ); }
		else if( s.getText() == "2" ) { this.setText( this.getText().concat("2") ); }
		else if( s.getText() == "3" ) { this.setText( this.getText().concat("3") ); }
		else if( s.getText() == "4" ) { this.setText( this.getText().concat("4") ); }
		else if( s.getText() == "5" ) { this.setText( this.getText().concat("5") ); }
		else if( s.getText() == "6" ) { this.setText( this.getText().concat("6") ); }
		else if( s.getText() == "7" ) { this.setText( this.getText().concat("7") ); }
		else if( s.getText() == "8" ) { this.setText( this.getText().concat("8") ); }
		else if( s.getText() == "9" ) { this.setText( this.getText().concat("9") ); }
		else if( s.getText() == "0" ) { this.setText( this.getText().concat("0") ); }
		else if( s.getText() == "." ) { this.setText( this.getText().concat(".") ); }
		else {
			setVisible(false);
			try {
				Integer result = null;
				float f = 0; // check
				switch(unit) {
				case INT:
					result = Integer.parseInt(textField.getText().trim());
					break;
				case BCDXPDR:
					if( textField.getText().trim().length() == 0 ) break;
					try {
						f= Float.parseFloat(textField.getText().trim());
					} catch( Exception e1 ) {}
					if( f == 7500 || f > 7777) {
						JOptionPane.showMessageDialog(this,
							    "Wrong transponder code, input supressed.",
							    "Wrong transponder code",
							    JOptionPane.ERROR_MESSAGE);
					} else {
						result = (int)(Float.parseFloat(textField.getText().trim()));  // 12350
						result = Integer.parseInt(""+result, 16);
					}
					break;
				case BCD16:
					if( textField.getText().trim().length() == 0 ) break;
					result = (int)(Float.parseFloat(textField.getText().trim() ));
					result = Integer.parseInt(""+result, 16);
					break;
				case BCD16NAVFREQ:
					if( textField.getText().trim().length() == 0 ) break;
					try {
						f = Float.parseFloat(textField.getText().trim());
					} catch( Exception e2 ) {}
					if( f < 108 || f >= 118 ) {
						JOptionPane.showMessageDialog(this,
							    "Enter a NAV frequency between 108.00 and 117.95",
							    "Wrong NAV frequency",
							    JOptionPane.ERROR_MESSAGE);
					} else {
						result = (int)(Float.parseFloat(textField.getText().trim()) * 100 % 10000 );  // 12350
						result = Integer.parseInt(""+result, 16);
					}
					break;
				case BCD16COMFREQ:
					if( textField.getText().trim().length() == 0 ) break;
					try {
						f = Float.parseFloat(textField.getText().trim());
					} catch( Exception e3 ) {}
					if( f < 118 || f >= 136.95 ) {
						JOptionPane.showMessageDialog(this,
							    "Enter a COM frequency between 118.00 and 136.95",
							    "Wrong COM Frequency",
							    JOptionPane.ERROR_MESSAGE);
					} else {
						result = (int)(Float.parseFloat(textField.getText().trim()) * 100 % 10000 );  // 12350
						result = Integer.parseInt(""+result, 16);
					}
					//result = result < 118 || result >= 137 ? null : result;
					break;
				case BCD16HZ: // OUTPUT: 6.9664768E7 = 4270000
					if( textField.getText().trim().length() == 0 ) break;
					try {
						f= Float.parseFloat(textField.getText().trim());
					} catch( Exception e4 ) {}
					if( f < 100 || f > 1799.9 ) {
						JOptionPane.showMessageDialog(this,
							    "Enter a ADF frequency between 100.00 and 1799.9",
							    "Wrong ADF frequency",
							    JOptionPane.ERROR_MESSAGE);
					} else {
						result = (int)(Float.parseFloat(textField.getText().trim()) * 10 );
						result = Integer.parseInt( ""+ ( result * 1000 ), 16 );
						Integer.reverseBytes(result);
					}
					break;
				case KOHLSMANINHG:
					//3,386.389
					if( textField.getText().trim().length() == 0 ) break;
					result = Integer.parseInt(textField.getText().trim()) * 16;
					break;
				case DEGREES:
					result = (int)(Integer.MAX_VALUE * Integer.parseInt(textField.getText().trim()) / 360);
					break;
				default:
					break;
				}
				if( result != null ) {
					this.owner.dispatchEvent(this.event, result);
				}
			} catch( NumberFormatException x ) { System.exit(0); }
		}
	}
	
	public void keyPressed(KeyEvent ke) {
		if( ke.getKeyCode() == KeyEvent.VK_ESCAPE ) {
			setVisible(false);
		}
		if( ke.getKeyCode() == KeyEvent.VK_ENTER ) {
			setVisible(false);
			this.actionPerformed(new ActionEvent(ke.getSource(),ke.getID(),ke.paramString()));
		}
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

}
