package org.gaugebook;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;

import javax.swing.JPanel;

public class LED extends JPanel {

	private static final long serialVersionUID = 1L;
	
	protected Color clrOff  = new Color(23,86,12);  // led off color
    protected Color clrOn = new Color(46,162,24); // led on color
    protected Color clrBorder = new Color(44,66,24); // border color
    String title = "";
    private boolean hidden = false;
    protected boolean on = false;
 
    public LED( String title ) {
    	this.title = title;
    	this.setPreferredSize( new Dimension( 20, 12 ));
    	this.setBackground( Color.DARK_GRAY );
    	this.setFont( this.getFont().deriveFont( (float)(this.getFont().getSize() * .8 )));
    }

    public void set( boolean on ) {
    	this.on = on;
    	repaint();
    }
    
    public void set( Double on ) {
    	this.on = on == 1;
    	repaint();
    }
    
    public void hide() {
    	hidden = true;
    	repaint();
    }
    
    public void show() {
    	hidden = false;
    	repaint();
    }
    
    protected void paintComponent( Graphics g ) {
        int w = getWidth();
        int h = getHeight();

        Rectangle rect = new Rectangle( 0, 0, w, h );
        Graphics2D g2 = (Graphics2D)g;
        if( !hidden ) {
	        g2.setColor( on ? clrOn : clrOff );
	        g2.fill( rect );
	        rect = new Rectangle( 0, 0, w-1, h-1 );
	        g2.setColor( clrBorder );
	        g2.draw( rect );
        } else {
	        g2.setColor( this.getBackground() );
	        g2.fill( rect );
        }
        FontMetrics fm = g.getFontMetrics();
        int sw = fm.stringWidth( title );
		int sh = fm.getMaxAscent(); //  - fm.getMaxDescent();
        double x = w/2 - sw/2;
        g2.setColor( Color.LIGHT_GRAY );
        g2.drawString( this.title, (float)x, (float)sh );
        g2.dispose();
    }
}
