package org.gaugebook;

import java.awt.Color;
import java.awt.Paint;
import java.awt.PaintContext;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.ColorModel;

import org.gaugebook.RoundGradientContext;

public class RoundGradientPaint implements Paint {
	protected Point2D point;
	protected Point2D mRadius;
	protected Color mPointColor, mBackgroundColor;

	public RoundGradientPaint(Point2D center, Color pointColor, Point2D radius, Color backgroundColor) {
		double x = center.getX();
		double y = center.getY();
		if (radius.distance(0, 0) <= 0)
			throw new IllegalArgumentException("Radius must be greater than 0.");
		point = new Point2D.Double(x, y);
		mPointColor = pointColor;
		mRadius = radius;
		mBackgroundColor = backgroundColor;
	}

	public PaintContext createContext(ColorModel cm, Rectangle deviceBounds,
			Rectangle2D userBounds, AffineTransform xform, RenderingHints hints) {
		Point2D transformedPoint = xform.transform(point, null);
		Point2D transformedRadius = xform.deltaTransform(mRadius, null);
		return new RoundGradientContext(transformedPoint, mPointColor,
		transformedRadius, mBackgroundColor);
	}

	public int getTransparency() {
		int a1 = mPointColor.getAlpha();
		int a2 = mBackgroundColor.getAlpha();
		return (((a1 & a2) == 0xff) ? OPAQUE : TRANSLUCENT);
	}
}

