package org.gaugebook;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;
import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Properties;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;


public class SettingsDialog extends JDialog {

	static final int RESULT_OK = 1;
	static final int RESULT_CANCEL = 0;
	
	JTextField tfIp = null;
	JTextField tfPort = null;
	JTextField tfFile = null;
	
	JFileChooser fileDialog = null;
	
	static private Integer[] vFPS = { 5, 10, 15, 20, 25, 30 };  
	
	JCheckBox cbAlwaysOnTop  = new JCheckBox("Always on top", true);
	JCheckBox cbFullscreen   = new JCheckBox("Fullscreen");
	JCheckBox cbSaveSettings = new JCheckBox("Save settings", true);
	JCheckBox cbSkipDialog   = new JCheckBox("Skip this Dialog next time");
	JCheckBox cbAntialias	 = new JCheckBox("Antialias");
	JCheckBox cbRadioPanel   = new JCheckBox("Radio Panel");
	JCheckBox cbFKeyPanel	 = new JCheckBox("FKey Panel");
	JLabel    lblFPS         = new JLabel("FPS");
	JComboBox sbFPS			 = new JComboBox( vFPS );
	JLabel 	  lblFile		 = new JLabel("Gauge XML File");
	JButton   btnOpenFile 	 = new JButton("Open");
	JButton   btnQuit 	     = new JButton("Quit");
	JButton   btnCancel      = new JButton("Cancel");
	JButton   btnConnect     = new JButton("Connect");
	
	ActionListener alCancelExit;
	Properties origProps = null;
	GaugeBook owner = null;
	
	boolean runMode = false;
	boolean doRestart = false;
	
	int result = RESULT_CANCEL;

	public SettingsDialog( final GaugeBook owner, String title ) {
		super( owner, title, true );
		this.owner = owner;
		
		//cbFullscreen.setEnabled(false);
		cbSaveSettings.setEnabled(false);
		
		fileDialog = new JFileChooser();
		fileDialog.setCurrentDirectory(new File(owner.getXmlPath()));
		fileDialog.setFileFilter(new FileFilter() {
			@Override
			public boolean accept(File f) {
				// TODO Auto-generated method stub
				String ext = null;
		        String s = f.getName();
		        int i = s.lastIndexOf('.');

		        if (i > 0 &&  i < s.length() - 1) {
		            ext = s.substring(i+1).toLowerCase();
		        }
		        return ext != null && ext.equals("xml");
			}
            public String getDescription() {
                return "Gauge XML Files (*.xml)";
            }
		});
		fileDialog.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileDialog.setAcceptAllFileFilterUsed(false);

		JPanel pnl1 = new JPanel( new GridLayout(7,2) );
        pnl1.setBorder( BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder( owner.appName +" Settings" ),
                BorderFactory.createEmptyBorder( 8, 8, 8, 8 )));
       	tfIp = new JTextField("192.168.178.21");
       	tfIp.setSize(200,20);

       	tfPort = new JTextField("9017");
       	tfPort.setSize(50,20);
       	
       	tfFile = new JTextField("C172.xml");
       	tfFile.setEditable(false);
       		
       	pnl1.add( new JLabel("IP: "));
       	pnl1.add( tfIp );
       	pnl1.add( new JLabel(" Port: "));
       	pnl1.add( tfPort );

       	pnl1.add( lblFile );
       	JPanel pnl3 = new JPanel( new GridLayout( 1, 2 ));
       	pnl3.add( tfFile );
       	pnl3.add( btnOpenFile );
       	pnl1.add( pnl3 );
       	btnOpenFile.addActionListener( new ActionListener() {
       		public void actionPerformed(ActionEvent e) {
       			int returnVal = fileDialog.showOpenDialog(owner);
       			//System.out.println("Action Performed!!");
       			if (returnVal == JFileChooser.APPROVE_OPTION) {
       	            File file = fileDialog.getSelectedFile();
		       	 	if( !file.equals(origProps.getProperty("gaugebook.xmlfile"))) {
		       	 		tfFile.setText(file.getName());
		       	 		doRestart = true;
		       	 	}
       	        }
       		}
       	});
       	
       	cbFKeyPanel.addActionListener( new ActionListener() {
       		public void actionPerformed(ActionEvent e) {
       			doRestart = true;
       		}
       	});
       	
       	cbRadioPanel.addActionListener( new ActionListener() {
       		public void actionPerformed(ActionEvent e) {
       			doRestart = true;
       		}
       	});

       	cbFullscreen.addActionListener( new ActionListener() {
       		public void actionPerformed(ActionEvent e) {
       			doRestart = true;
       		}
       	});

       	pnl1.add( cbAlwaysOnTop );
       	pnl1.add( cbFullscreen );
       	pnl1.add( cbRadioPanel );
       	pnl1.add( cbFKeyPanel );
       	pnl1.add( cbSaveSettings );
       	pnl1.add( cbSkipDialog );
       	pnl1.add( cbAntialias );
       	
       	JPanel pnlFps = new JPanel( new GridLayout(1,2));
       	pnlFps.add( lblFPS );
       	pnlFps.add( sbFPS );
       	pnl1.add(pnlFps);
       	sbFPS.setSelectedIndex(1);
       	
       	btnConnect = new JButton("Connect");
       	btnConnect.addActionListener(new ActionListener() {
    	   public void actionPerformed(ActionEvent e) {
    		   	result = RESULT_OK;
    			setVisible(false);

			}});

       	alCancelExit = new ActionListener() {
      	   public void actionPerformed(ActionEvent e) {
      		   setVisible( false );
      		   result = RESULT_CANCEL;
      		   owner.shutdown();
      	   }};

        this.addKeyListener( new KeyAdapter() {
      	   	public void keyPressed(KeyEvent e) {
      	   		if( e.getKeyCode() == KeyEvent.VK_ESCAPE ) {
           		   result = RESULT_CANCEL;
      			   setVisible(false);
      	   		}
      	   	}
    	});
      	   
       	btnCancel.addActionListener( alCancelExit );
       	btnQuit.addActionListener( alCancelExit );
       	
       	// todo: keyboard shortcuts

       	
        JPanel pnl2 = new JPanel( new FlowLayout( FlowLayout.RIGHT, 4, 4 ));
        pnl2.add( btnConnect );
        pnl2.add( btnCancel  );
        pnl2.add( btnQuit );
        
        // bag.setConstraints( tmp , con );
        // pane.add( tmp );

        getContentPane().setLayout( new BorderLayout( 4, 4 ));
        getContentPane().add( pnl1, BorderLayout.CENTER );
        getContentPane().add( pnl2, BorderLayout.SOUTH );
        
        pack();

        // set location on screen
        if( owner.isVisible() ) {
        	Dimension size = owner.getSize();
        	Point loc = owner.getLocationOnScreen();
        	setLocation( loc.x, loc.y );
	        setLocation(( loc.x + ( size.width - getSize().width )) / 2 ,
                        ( loc.y + ( size.height - getSize().height )) / 2 );
        }
        else {
	        Dimension size = Toolkit.getDefaultToolkit().getScreenSize();
	        setLocation(( size.width - getSize().width ) / 2 ,
	                    ( size.height - getSize().height ) / 2 );
        }
        setResizable( false );
        
	}
	
    public void setProperties( Properties props ) {
    	origProps = props;
        tfIp.setText( (String)props.getProperty( "gaugebook.connect.ip" ));
        tfPort.setText( (String)props.getProperty( "gaugebook.connect.port" ));
        tfFile.setText( (String)props.getProperty( "gaugebook.xmlfile" ));
        if( props.containsKey("gaugebook.alwaysontop"))
        	cbAlwaysOnTop.setSelected( ((String)props.getProperty( "gaugebook.alwaysontop" )).equals( "true" ));
        if( props.containsKey("gaugebook.fullscreen"))
        	cbFullscreen.setSelected( ((String)props.getProperty( "gaugebook.fullscreen" )).equals( "true" ));
        if( props.containsKey("gaugebook.savesettings"))
        	cbSaveSettings.setSelected( ((String)props.getProperty( "gaugebook.savesettings" )).equals( "true" ));
        if( props.containsKey("gaugebook.skipdialog"))
        	cbSkipDialog.setSelected( ((String)props.getProperty( "gaugebook.skipdialog" )).equals("true"));
        if( props.containsKey("gaugebook.antialias"))
        	cbAntialias.setSelected( ((String)props.getProperty( "gaugebook.antialias" )).equals("true"));
        if( props.containsKey("gaugebook.radiopanel"))
        	cbRadioPanel.setSelected( ((String)props.getProperty( "gaugebook.radiopanel" )).equals("true"));
        if( props.containsKey("gaugebook.fkeypanel"))
        	cbFKeyPanel.setSelected( ((String)props.getProperty( "gaugebook.fkeypanel" )).equals("true"));
        if( props.containsKey("gaugebook.fps"))
        	sbFPS.setSelectedItem((Integer)Integer.parseInt(props.getProperty("gaugebook.fps")));
        if( props.containsKey("gaugebook.height") && props.containsKey("gaugebook.width"))
			owner.setSize( (int)Float.parseFloat(props.getProperty("gaugebook.width")), 
						   (int)Float.parseFloat(props.getProperty("gaugebook.height")));
        if( props.containsKey("gaugebook.xpos") && props.containsKey("gaugebook.ypos"))
			owner.setLocation((int)Float.parseFloat(props.getProperty("gaugebook.xpos")), 
			 		 		  (int)Float.parseFloat(props.getProperty("gaugebook.ypos")));
    }

    public Properties getProperties() {
        Properties props = new Properties();
        props.setProperty( "gaugebook.connect.ip", tfIp.getText() );
        props.setProperty( "gaugebook.connect.port", tfPort.getText() );
        props.setProperty( "gaugebook.xmlfile", tfFile.getText() );
        props.setProperty( "gaugebook.alwaysontop", cbAlwaysOnTop.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.fullscreen", cbFullscreen.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.savesettings", cbSaveSettings.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.skipdialog", cbSkipDialog.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.antialias", cbAntialias.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.radiopanel", cbRadioPanel.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.fkeypanel", cbFKeyPanel.isSelected() ? "true" : "false" );
        props.setProperty( "gaugebook.fps", ""+ sbFPS.getSelectedItem() );
        props.setProperty( "gaugebook.width", ""+owner.getSize().getWidth());
        props.setProperty( "gaugebook.height", ""+owner.getSize().getHeight());
        props.setProperty( "gaugebook.xpos", ""+owner.getLocation().getX());
        props.setProperty( "gaugebook.ypos", ""+owner.getLocation().getY());
        props.setProperty( "gaugebook.licenseaccepted", ""+owner.licenseaccepted );
        return props;
    }
	
    public void setVisible( boolean isVisible ) {
    	this.setLocationRelativeTo(owner);
    	super.setVisible( isVisible );
    }
    
    public void setRunMode( boolean runMode ) {
    	if( runMode ) {
	    	runMode = true;
	    	btnConnect.setText("Ok");
	    	btnCancel.removeActionListener(alCancelExit);
	       	btnCancel.addActionListener(new ActionListener() {
	     	   public void actionPerformed(ActionEvent e) {
	     		    if( origProps != null ) setProperties(origProps);
	 				setVisible(false);
	 			}});
    	}
	}
    
    public int getResult() {
    	return result;
    }
    
	public String getIP() {
		return this.tfIp.getText();
	}
	
	public void setIP(String text) {
		this.tfIp.setText(text);
	}
	
	public int getPort() {
		return Integer.parseInt(this.tfPort.getText());
	}
	
	public void setPort(int port) {
		this.tfPort.setText(""+ port);
	}

	public Properties getDefaultProperties() {
        Properties props = new Properties();
        String ip = "127.0.0.1";
        try {
			ip = InetAddress.getLocalHost().getHostAddress();
		} catch (UnknownHostException e) {}
        props.setProperty( "gaugebook.connect.ip", ip );
        props.setProperty( "gaugebook.connect.port", "9017" );
        props.setProperty( "gaugebook.alwaysontop", "false" );
        props.setProperty( "gaugebook.fullscreen", "false" );
        props.setProperty( "gaugebook.savesettings", "true" );
        props.setProperty( "gaugebook.skipdialog", "true" );
        props.setProperty( "gaugebook.antialias", "true" );
        props.setProperty( "gaugebook.radiopanel", "false" );
        props.setProperty( "gaugebook.fkeypanel", "true" );
        props.setProperty( "gaugebook.xmlfile", "KODI.xml" );
        props.setProperty( "gaugebook.fps", "20" );
        props.setProperty( "gaugebook.width", ""+owner.originalWidth );
        props.setProperty( "gaugebook.height", ""+owner.originalHeight );
        props.setProperty( "gaugebook.xpos", ""+owner.getLocation().getX());
        props.setProperty( "gaugebook.ypos", ""+owner.getLocation().getY());
        props.setProperty( "gaugebook.licenseaccepted", "false");
        return props;
	}
}
