package org.gaugebook;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JPanel;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XmlGuiBuilder {
	
	private static void addGauge(GaugeBook owner, Element xmlElmnt, JPanel pnl, Vector<AbstractGauge> gauges ) {
		NodeList gaugeLst = xmlElmnt.getChildNodes();
		if( owner.debug ) System.out.println("Add Gauge: "+ xmlElmnt.getAttribute("class") +" gaugeLst.len: "+ gaugeLst.getLength());
		
		try {
			Class cl = Class.forName(xmlElmnt.getAttribute("class"));
			Object obj = cl.newInstance(); 
			if( obj instanceof AbstractGauge ) {
				AbstractGauge gauge = (AbstractGauge)GaugeManager.load(xmlElmnt.getAttribute("class"));
				
				for (int s = 0; s < gaugeLst.getLength(); s++) 
				{
					Node pnlNode = gaugeLst.item(s);
					if (pnlNode.getNodeType() == Node.ELEMENT_NODE) 
					{
						Element elmnt = (Element)pnlNode;
						if( elmnt.getTagName() == "property" ) {
							if( owner.debug ) System.out.println("--- set property: "+elmnt.getAttribute("name")+" = "+elmnt.getAttribute("value"));
							gauge.setProperty( elmnt.getAttribute("name"), elmnt.getAttribute("value"));
						}
					}
				}

				gauge.init( owner, xmlElmnt.getAttribute("title"));
				gauge.updateProperties();
				gauges.add(gauge);
				pnl.add(gauge);
				gauge = null;
			}
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	private static void addPanel(GaugeBook owner, Element pnlElmnt, JPanel pnl, Vector<JPanel> panels, Vector<AbstractGauge> gauges ) {
		NodeList pnlLst = pnlElmnt.getChildNodes();		
		if( owner.debug ) System.out.println("Add Panel: "+ pnlElmnt.getAttribute("name") +" pnlLst.len: "+ pnlLst.getLength());
		pnl.setBackground(Color.DARK_GRAY);
		if( pnlElmnt.getAttribute("layout").equals("GridLayout")) {
			pnl.setLayout( new GridLayout(Integer.parseInt(pnlElmnt.getAttribute("rows")), 
										  Integer.parseInt(pnlElmnt.getAttribute("cols"))));
		} else if( pnlElmnt.getAttribute("layout").equals("GridBagLayout")) {
			pnl.setLayout( new GridBagLayout() );
		} else if( pnlElmnt.getAttribute("layout").equals("BorderLayout")) {
			BorderLayout bl = new BorderLayout(4,4);
			bl.preferredLayoutSize(owner);
			pnl.setLayout( bl );
			bl.invalidateLayout(owner);
		}

		for (int s = 0; s < pnlLst.getLength(); s++) 
		{
			Node pnlNode = pnlLst.item(s);
			if (pnlNode.getNodeType() == Node.ELEMENT_NODE) 
			{
				Element elmnt = (Element)pnlNode;
				if( elmnt.getTagName() == "panel" ) {
					JPanel newPanel = new JPanel();
					if( elmnt.getAttribute("borderlayout").equals("WEST")) {
						pnl.add(newPanel, BorderLayout.WEST);
						if( owner.debug ) System.out.println("- BorderLayout.WEST");
					}
					else if( elmnt.getAttribute("borderlayout").equals("CENTER")) {
						pnl.add(newPanel, BorderLayout.CENTER);
						if( owner.debug ) System.out.println("- BorderLayout.CENTER");
					}
					else if( elmnt.getAttribute("borderlayout").equals("EAST")) {
						pnl.add(newPanel, BorderLayout.EAST);
						if( owner.debug ) System.out.println("- BorderLayout.EAST");
					}
					else {
						pnl.add(newPanel);
						if( owner.debug ) System.out.println("- GridLayout");
					}
					addPanel( owner, elmnt, newPanel, panels, gauges );
				}
				else if( elmnt.getTagName() == "gauge") {
					addGauge( owner, elmnt, pnl, gauges );
				}
			}
		}
	}
	
	public static void loadGui( GaugeBook owner, String filename, Vector<JPanel> panels, Vector<AbstractGauge> gauges ) 
			throws ParserConfigurationException, SAXException, IOException 
	{
		File file = new File( filename );
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder db = dbf.newDocumentBuilder();
		Document doc = db.parse(file);
		doc.getDocumentElement().normalize();
		NodeList rootLst = doc.getElementsByTagName("panels");
		Element rootElmnt = (Element)rootLst.item(0);
		NodeList pnlLst = rootElmnt.getChildNodes();

		if( owner.debug ) System.out.println("Root element " + doc.getDocumentElement().getNodeName() + " Len: "+pnlLst.getLength());
		for (int s = 0; s < pnlLst.getLength(); s++) 
		{
			Node pnlNode = pnlLst.item(s);
			if (pnlNode.getNodeType() == Node.ELEMENT_NODE) 
			{
				Element pnlElmnt = (Element) pnlNode;
				JPanel pnl = new JPanel();
				panels.add(pnl);
				addPanel(owner, pnlElmnt, pnl, panels, gauges);
			}
		}
		if( owner.debug ) System.out.println("Gauges: "+gauges.size() +" Panels: "+ panels.size() );
	}
	
}