package org.gaugebook.gauges.aviation.generic;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;

/**
 * Generic RPM Gauge
 * 
 * @author Michael Nagler
 */
public class GenericRPM extends AbstractClockGauge {
	
	private FSXVariable rpmVar;

	private int index;
	
	private double rpm = 0;
	
	@Override
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
        int index = Integer.parseInt( getProperty("engineIndex").toString());
        if( index < 1 || index > 4 ) index = 1;
		this.setMinValue( 0 );
        this.setMaxValue( 35 );
        this.setText("RPM x 100");
        this.setDecPlaceCount(0);
        this.setMajorTickStep(5);
        this.setMinorTickStep(1);
        this.setAngleExtend(270);
        this.setAngleStart(-45);
        this.setValue(0);
        this.setShowValue(false);
	    super.setTextPosition(0, -.25);
        // this.index = index;
	    rpmVar = owner.registerFSXVariable( "GENERAL ENG RPM:"+ index );
    }
	
	public void update() {
		rpm = rpmVar.getDoubleValue();
		setValue( rpm );
		repaint();
	}
	
	protected void drawNeedle( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
		super.drawNeedle(r, needleR, cp, g2, rpm/100);
	}
		   
	protected void drawScale( final double r, final Point2D cp, Graphics2D g2 ) {
    	super.drawArc( 20, 26, new Color(0,140,0), r, .85, .9, cp, g2 );
    	super.drawScale(r, cp, g2);
    	super.drawMarker(26, new Color(160,0,0), 3, r, .8, .9, cp, g2);
    }
}
