package org.gaugebook.gauges.aviation.simple;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;

import javax.print.attribute.IntegerSyntax;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;


/**
 * Airspeed Indicator Gauge
 * 
 *  
 * @author Nagel
 *
 */
public class AirspeedIndicator extends AbstractClockGauge {

	private FSXVariable indicatedAirspeed;
	private FSXVariable trueAirspeed;
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
		
		this.setMinValue( 40 );
        this.setMaxValue( 200 );
        this.setText("Airspeed");
        this.setDecPlaceCount(0);
        this.setMajorTickStep(20);
        this.setMinorTickStep(5);
        this.setValue(0);
        this.setAngleStart(125);
        this.setAngleExtend(290);

        super.setTextPosition( 0, -.64);
        indicatedAirspeed = owner.registerFSXVariable("AIRSPEED INDICATED"); 
        trueAirspeed = owner.registerFSXVariable("AIRSPEED TRUE");
	}
	
	@Override
	protected void drawValue( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
		super.setValue(indicatedAirspeed.getDoubleValue());
		super.drawValue(r, needleR, cp, g2);
        super.drawText(r, cp, g2, new Point2D.Double(.02,-.2), 1, "kts");
	}
	
    protected void drawScale( final double r, final Point2D cp, Graphics2D g2 ) {
    	if( getProperty("greenArc") != null ) {
    		String[] args = ((String)getProperty("greenArc")).split(",");
    		super.drawArc( Integer.parseInt(args[0]), Integer.parseInt(args[1]), new Color(0,160,0), r, .86, .94, cp, g2 );
    	}
    	if( getProperty("yellowArc") != null ) {
    		String[] args = ((String)getProperty("yellowArc")).split(",");
    		super.drawArc( Integer.parseInt(args[0]), Integer.parseInt(args[1]), new Color(160,160,0), r, .86, .94, cp, g2 );
    	}
    	if( getProperty("whiteArc") != null ) {
    		String[] args = ((String)getProperty("whiteArc")).split(",");
    		super.drawArc( Integer.parseInt(args[0]), Integer.parseInt(args[1]), new Color(160,160,160), r, .90, .94, cp, g2 );
    	}

    	super.drawScale(r, cp, g2);

    	if( getProperty("redMarker") != null ) {
    		String[] args = ((String)getProperty("redMarker")).split(",");
    		super.drawMarker( Integer.parseInt(args[0]), new Color(160,0,0), 4, r, .82, .92, cp, g2 );
    	}
    }
}
