package org.gaugebook.gauges.aviation.simple;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;
import org.gaugebook.math.Angle;


/**
 * Attitude Indicator 
 * 
 * @author Michael Nagler
 *
 */
public class AttitudeIndicatorAerobatics extends AbstractClockGauge {

	private static final long serialVersionUID = 1L;
	
	private FSXVariable bankVar;
	private FSXVariable pitchVar;

	private double bank=0, pitch=0;

	private static final Color skyColorAI = new Color(128,128,255);
	private static final Color gndColorAI = new Color(164,96,48);
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
		this.setMinValue( -90 );
        this.setMaxValue( 90 );
        this.setText("Attitude Indicator");
        this.setDecPlaceCount(0);
        this.setMajorTickStep(30);
        this.setMinorTickStep(10);
        this.setAngleExtend(180);
        this.setAngleStart(0);
        this.setValue(0);
	    super.setTextPosition(1.1, 0.42);
        
	    bankVar  = owner.registerFSXVariable("ATTITUDE INDICATOR BANK DEGREES");
	    pitchVar = owner.registerFSXVariable("ATTITUDE INDICATOR PITCH DEGREES");
	}

	public void update() {
		pitch = ( 180 / StrictMath.PI ) * pitchVar.getDoubleValue();
		bank = ( 180 / StrictMath.PI ) * bankVar.getDoubleValue();
		repaint();
	}

    protected synchronized void drawText( final double r, final Point2D cp, Graphics2D g2 ) {
    }
    
    protected synchronized void drawScale( final double r, final Point2D cp, Graphics2D g2 ) {
    	// draw lines every frame to get antialiasing - java 2d performance test :D
    	Shape oldClip = g2.getClip();
    	double f = .76;
    	Ellipse2D.Double ell = new Ellipse2D.Double( cp.getX() - r * f, cp.getY() - r * f, 2 * r * f, 2 * r * f );
		Point2D p1, p2, p3, p4;
    	Polygon pol;
		g2.setClip(ell);
		double pf = 40;
		double pv = pitch / pf;

        // sky
		g2.setColor(skyColorAI);
		g2.fillRect(0, 0, getWidth(), getHeight());

		// gnd
		Angle a = new Angle( -1 * bank );
		p1 = getPoint( cp, r * pv, a.getDegreesPlus(90) );
		p1 = getPoint( p1, r * .8, a.getDegrees() );
		p2 = getPoint( p1, r * 1.8, a.getDegreesPlus(180));
		p3 = getPoint( p2, r * 180 / pf, a.getDegreesPlus(270));
		p4 = getPoint( p1, r * 180 / pf, a.getDegreesPlus(270));
		pol = new Polygon();
        pol.addPoint( (int)Math.round(p1.getX()), (int)Math.round(p1.getY()) );
        pol.addPoint( (int)Math.round(p2.getX()), (int)Math.round(p2.getY()) );
        pol.addPoint( (int)Math.round(p3.getX()), (int)Math.round(p3.getY()) );
        pol.addPoint( (int)Math.round(p4.getX()), (int)Math.round(p4.getY()) );
		
		g2.setColor(gndColorAI);
		g2.fill(pol);
		
		// horizont line
		g2.setColor(Color.WHITE);
		g2.setStroke(new BasicStroke((int)Math.round(r*0.02)));
        g2.draw( new Line2D.Double(p1, p2));

        // 30 / 60 lines
        p1 = getPoint( cp, r * pv, a.getDegreesPlus(90));
        p2 = getPoint( p1, r * 1.6, a.getDegreesPlus(210));//210));
        p3 = getPoint (p1, r * 1.6, a.getDegreesPlus(240));//240));
		g2.setStroke(new BasicStroke((int)Math.round(r*0.015)));
		g2.draw( new Line2D.Double(p1, p2)); 
		g2.draw( new Line2D.Double(p1, p3)); 

		p2 = getPoint( p1, r * 1.6, a.getDegreesPlus(300));
        p3 = getPoint (p1, r * 1.6, a.getDegreesPlus(330));
		g2.draw( new Line2D.Double(p1, p2)); 
		g2.draw( new Line2D.Double(p1, p3)); 

        // scale
		g2.setColor(Color.WHITE);
		g2.setStroke(new BasicStroke((int)Math.round(r*0.01)));

		double step = 10;
		for( int i = -85; i < 90; i += step ) {
			p1 = getPoint( cp, r * (pitch + i ) / pf, a.getDegreesPlus(90) );
			p1 = getPoint( p1, r * .12, a.getDegrees() );
			p2 = getPoint( p1, r * .24, a.getDegreesPlus(180));
			g2.draw( new Line2D.Double(p1, p2)); 
    	}

		for( int i = -90; i <= 90; i += 10 ) {
			p1 = getPoint( cp, r * (pitch + i ) / pf, a.getDegreesPlus(90) );
			p1 = getPoint( p1, r * .2, a.getDegrees() );
			p2 = getPoint( p1, r * .4, a.getDegreesPlus(180));
			g2.draw( new Line2D.Double(p1, p2)); 
    	}

		// triangle
        p1 = getPoint( cp, r * .7, a.getDegreesPlus(270));
        p2 = getPoint( p1, r * .15, a.getDegreesPlus(60));
        p3 = getPoint (p1, r * .15, a.getDegreesPlus(120));
        
        pol = new Polygon();
        pol.addPoint( (int)Math.round(p1.getX()), (int)Math.round(p1.getY()) );
        pol.addPoint( (int)Math.round(p2.getX()), (int)Math.round(p2.getY()) );
        pol.addPoint( (int)Math.round(p3.getX()), (int)Math.round(p3.getY()) );

        g2.setColor( Color.WHITE );
        g2.fill( pol);

		g2.setClip(oldClip);
    }
    
    protected synchronized void drawNeedle( final double r, double needleR, final Point2D cp, Graphics2D g2 ) {
        // draw needle
    	
    	Angle angleCurrent;
    	Point2D pointLeft, pointRight;

        angleCurrent = new Angle( 270 );

        Point2D np;
        
        // draw horizonal lines
        g2.setStroke( new BasicStroke(2f) );   
        np = getPoint( cp, 0, angleCurrent.getDegrees() );
        pointLeft  = getPoint( np, needleR * .8, 0 );
        pointRight = getPoint( np, needleR * .4, 0 );
		g2.draw( new Line2D.Double( pointLeft, pointRight )); 

        pointLeft  = getPoint( np, needleR * -.8, 0 );
        pointRight = getPoint( np, needleR * -.4, 0 );
		g2.draw( new Line2D.Double( pointLeft, pointRight )); 

		g2.setStroke( new BasicStroke(4f) );   
        g2.draw( new Line2D.Double( np, np ));
    }

    protected void drawValue( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
    	Shape oldClip = g2.getClip();
    	double f = .73;
    	Ellipse2D.Double ell = new Ellipse2D.Double( cp.getX() - r * f, cp.getY() - r * f, 2 * r * f, 2 * r * f );
        
        f = .76;
    	ell = new Ellipse2D.Double( cp.getX() - r * f, cp.getY() - r * f, 2 * r * f, 2 * r * f );
    	
		g2.setClip(ell);
        g2.setColor(Color.GRAY);
        g2.draw(ell);
        
        f = .93;
    	Ellipse2D.Double ell1 = new Ellipse2D.Double( cp.getX() - r * f, cp.getY() - r * f, 2 * r * f, 2 * r * f );
    	Area area = new Area( ell1 );
    	area.subtract( new Area( ell ));
    	
    	g2.setClip(area);

         // top scale lines
        g2.setColor( Color.WHITE );
        Angle angleCurrent;
        Point2D ap, np;
        g2.setStroke( new BasicStroke(2f));
        for( int a = -90; a <= 90; a += 10 ) {
        	if( Math.abs(a)==80 || Math.abs(a)==70 || Math.abs(a)==40 || Math.abs(a)==50 ) continue;
        	f = Math.abs(a) == 10 || Math.abs(a) == 20 ? .84 : .93;
            angleCurrent = new Angle( a + 270 ); // new Angle( bank + 90 + -1 * bank + a );
            ap = getPoint( cp, r * .74, angleCurrent.getDegrees() );
            np = getPoint( cp, r * f, angleCurrent.getDegrees() );
            g2.draw( new Line2D.Double(ap, np));
        }

        g2.setClip(oldClip);
    }
}
