package org.gaugebook.gauges.aviation.simple;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;
import org.gaugebook.math.Angle;


/**
 * VOR Compass with GlideScope
 * 
 * todo: buffer to / from / off flags
 * 
 * @author Michael Nagler
 *
 */
public class CourseDeviationIndicatorGS extends CourseDeviationIndicator {

	private FSXVariable vorGSAngle, vorGSFlag;
	private int index;
	private double gsa, gsf;

	public void init( GaugeBook owner, String title, int index ) {
		super.init( owner, title, index );
		
		vorGSAngle = owner.registerFSXVariable("NAV GLIDE SLOPE ERROR:"+ index);
		vorGSFlag = owner.registerFSXVariable("NAV GS FLAG:"+index);
	}
	
	public void init( GaugeBook owner, String title ) {
		init( owner, title, 1);
	}
	
	@Override
	public void update() {
		if( gsa != vorGSAngle.getDoubleValue()
		 || gsf != vorGSFlag.getDoubleValue() ) {
		 	gsa = vorGSAngle.getDoubleValue();
			gsf = vorGSFlag.getDoubleValue();
		}
		repaint();
	}
	
	@Override
	protected void drawBackground( final double r, final Point2D cp, final Graphics2D g2 ) {
		super.drawBackground(r, cp, g2);
		float f = (float)(r *.03);
		float dash[] = { f };
		g2.setStroke(new BasicStroke( f, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER, f, dash, 0.0f));
		g2.setColor(Color.WHITE);
		Point2D p0 = getPoint(cp, r * .55, 0);
		Point2D p1 = getPoint(cp, r * .55, 180);
		g2.draw( new Line2D.Double( p0, p1 ));
		
	}
	
	@Override
	protected void drawNeedle( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
		this.setMinValue( -128 );
        this.setMaxValue(  128 );
        this.setAngleExtend(80);
        this.setAngleStart(230);

        Point2D p0, p1, p2, p3, p4;
    	Rectangle2D rect;
    	Shape oldClip = g2.getClip();

    	// gs flag
        // drawText( final double r, final Point2D cp, Graphics2D g2, Point2D position, double textHeight, String text )
        super.drawText(r, cp, g2, new Point2D.Double(0.15,-0.015),1,"GS");
    	p1 = getPoint(cp, r*.28, 30);
    	p2 = getPoint(p1, r*.20, -26);
    	g2.setColor(Color.WHITE);
    	rect = new Rectangle2D.Double(p1.getX(),p1.getY(),p2.getX()-p1.getX(),p2.getY()-p1.getY());
    	g2.fill(rect);
        if( gsf == 1.0 ) {
        	// draw ON
        	g2.setColor(Color.BLACK);
            super.drawText(r, cp, g2, new Point2D.Double(0.335,-0.025),0.9,"ON");
        }
        else {
        	g2.fill(rect.getBounds2D());
        	g2.clip(rect.getBounds2D());
        	g2.setColor(Color.RED);
        	g2.setStroke(new BasicStroke((float)(r*.04)));
        	p1 = getPoint(cp, r*.16, 16);
        	p2 = getPoint(p1, r*.15, 45);
        	for( int i = 0; i < 3; i ++ ) {
    	    	g2.draw(new Line2D.Double(p1, p2));
        		p1.setLocation( p1.getX()+r*.12, p1.getY());
        		p2.setLocation( p2.getX()+r*.12, p2.getY());
            }
            g2.setClip(oldClip);
        }
        
        // cdi needle
        Point2D mp = getPoint( cp, r*.6, 90 );
        double tickRate = 25.6;
        double myValue = -1 * cdi;
        if( myValue < minValue || myValue > maxValue ) { 
        	myValue = (myValue < minValue ? minValue : maxValue);
        }
        double angleCurrent = angleStart + angleExtend * (1.0 - (myValue - minValue)/(maxValue - minValue));

        Point2D ap = getPoint( mp, needleR * 1.5, angleCurrent );
        
        g2.setColor( Color.WHITE );
        g2.setStroke( new BasicStroke(2f));
        g2.draw( new Line2D.Double(mp, ap));
        
        // cdi scale
        g2.setStroke( new BasicStroke(2f));
        p1 = getPoint( cp, r * .65, 180 );
        p2 = getPoint( cp, r * .65, 0 );
    	g2.setColor( Color.RED);
        Line2D.Double line = new Line2D.Double( p1, p2 );
    	g2.setClip(line);
    	g2.setColor( minorTickColor );
        // double minorGradSteps = angleExtend * minorTickStep / (maxValue - minValue);
        double stepAngle = angleExtend * tickRate/(maxValue-minValue);
        for( double v = minValue; v<= maxValue; v+=tickRate) {
            Angle currentAngle = new Angle( angleStart + angleExtend - stepAngle * (v - minValue) / tickRate );
            p0 = getPoint( mp, r * 0.5, currentAngle.getDegrees() );
            p1 = getPoint( mp, r * 0.75, currentAngle.getDegrees() );
            g2.draw( new Line2D.Double( p0, p1 ) );
        }
        g2.setClip(oldClip);

        // draw to / from triangle
        // 0 = Off, 1 = TO, 2 = FROM
        Polygon pol;
        switch( (int)tf ) {
        case 1: // to
            pol = new Polygon();
        	p1 = getPoint(cp, r*.55, 271);
        	p2 = getPoint(cp, r*.65, 260);
        	p3 = getPoint(cp, r*.65, 280);
        	pol.addPoint((int)p1.getX(),(int)p1.getY());
        	pol.addPoint((int)p2.getX(),(int)p2.getY());
        	pol.addPoint((int)p3.getX(),(int)p3.getY());
        	g2.fill(pol);
        	break;
        case 2: // from
            pol = new Polygon();
        	p1 = getPoint(cp, r*.65, 270);
        	p2 = getPoint(cp, r*.55, 260);
        	p3 = getPoint(cp, r*.55, 280);
        	pol.addPoint((int)p1.getX(),(int)p1.getY());
        	pol.addPoint((int)p2.getX(),(int)p2.getY());
        	pol.addPoint((int)p3.getX(),(int)p3.getY());
        	g2.fill(pol);
        	break;
        default: // off or error 
        	p1 = getPoint(cp, r*.55, 260);
        	p2 = getPoint(cp, r*.65, 280);
        	rect = new Rectangle2D.Double(p1.getX(),p1.getY(),p2.getX()-p1.getX(),p2.getY()-p1.getY());
        	g2.fill(rect);
	    	g2.clip(rect.getBounds2D());
	    	g2.setColor(Color.RED);
	    	g2.setStroke(new BasicStroke((float)(r*.04)));
	    	p1 = getPoint(cp, r*.67, 255);
	    	p2 = getPoint(p1, r*.15, 45);
	    	for( int i = 0; i < 3; i ++ ) {
		    	g2.draw(new Line2D.Double(p1, p2));
	    		p1.setLocation( p1.getX()+r*.12, p1.getY());
	    		p2.setLocation( p2.getX()+r*.12, p2.getY());
	        }
	        g2.setClip(oldClip);
        }

        // gs needle
        this.setAngleExtend(50);
        this.setAngleStart(-25);
		this.setMinValue( -2 );
        this.setMaxValue(  2 );

        mp = getPoint( cp, r*.6, 180 );
        tickRate = 1;
        myValue = gsa;
        if( myValue < minValue || myValue > maxValue ) { 
        	myValue = (myValue < minValue ? minValue : maxValue);
        }
        angleCurrent = angleStart + angleExtend * (1.0 - (myValue - minValue)/(maxValue - minValue));

        ap = getPoint( mp, needleR * 1.7, angleCurrent );
        
        g2.setColor( Color.WHITE );
        g2.setStroke( new BasicStroke(2f));
        g2.draw( new Line2D.Double(mp, ap));
        
        // gs scale
        g2.setColor( minorTickColor );
        g2.setStroke( new BasicStroke(2f));
        stepAngle = angleExtend * tickRate/(maxValue-minValue);
/*
        p1 = getPoint(cp, r*.6, 91);
        p2 = getPoint(cp, r*.6, 27);
    	rect = new Rectangle2D.Double(p1.getX(),p1.getY(),p2.getX()-p1.getX(),p2.getY()-p1.getY());
    	g2.fill(rect.getBounds2D());
        for( double v = minValue; v<= maxValue; v+=tickRate) {
            Angle currentAngle = new Angle( angleStart + angleExtend - stepAngle * (v - minValue) / tickRate );
            p0 = getPoint( mp, r*.5, currentAngle.getDegrees() );
            p1 = getPoint( mp, r*.7, currentAngle.getDegrees() );
            g2.draw( new Line2D.Double( p0, p1 ) );
        }
*/
        for( double v = minValue; v<= maxValue; v+=tickRate) {
            Angle currentAngle = new Angle( angleStart + angleExtend - stepAngle * (v - minValue) / tickRate );
            p0 = getPoint( mp, r*1, currentAngle.getDegrees() );
            p1 = getPoint( mp, r*1.04, currentAngle.getDegrees() );
            g2.draw( new Line2D.Double( p0, p1 ) );
        }

        // reset values
        this.setAngleExtend(360);
        this.setAngleStart(270);
		this.setMinValue( 0 );
        this.setMaxValue( 359 );
        this.setMajorTickStep(30);
        this.setMinorTickStep(10);

		// n, w, s, e markers
		g2.setColor( Color.ORANGE );
        double fo = .93;
    	Ellipse2D.Double ello = new Ellipse2D.Double( cp.getX() - r * fo, cp.getY() - r * fo, 2 * r * fo, 2 * r * fo );
		g2.setClip(ello);
        for( double a = 0; a < 360; a += 90 ) {
            pol = new Polygon();
        	p1 = getPoint( cp, r * .85, new Angle( a ).getDegrees() );
        	p2 = getPoint( p1, r * .2, new Angle( a +30 ).getDegrees() );
        	p3 = getPoint( p1, r * .2, new Angle( a -30 ).getDegrees() );
        	pol.addPoint((int)p1.getX(),(int)p1.getY());
        	pol.addPoint((int)p2.getX(),(int)p2.getY());
        	pol.addPoint((int)p3.getX(),(int)p3.getY());
        	g2.fill(pol);
        }

        g2.setClip(oldClip);
	}
    
	@Override
    protected void drawValue( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
        // System.out.println("VOR"+index+":"+obs);
        //g2.rotate( Math.PI * (360 - cobs) / 180, cp.getX(), cp.getY() );
    }
    @Override
    protected void drawText( final double r, final Point2D cp, Graphics2D g2, Point2D position, double textHeight, String text ) {
    	
    }
}
