package org.gaugebook.gauges.aviation.simple;

import java.awt.Graphics2D;
import java.awt.geom.Point2D;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;
import org.gaugebook.AbstractClockGauge.Direction;


/**
 * Radar Altimeter Gauge
 * 
 * The radar alt needs 2 scales and therefore will be rewritten as soon as the gauge class
 * design has been improved...
 * 
 * @author Michael Nagler
 */
public class RadarAltimeter extends AbstractClockGauge {
	
	private FSXVariable radarAltitude;
	
	private double ialt; // indicated altitude value
	private double cialt; // current indicated altitude as shown by the gauge 
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, "RADAR ALT" );
		setText("RADAR ALT");
		// setUnitText("ft x 100");
	    setTextPosition(0, -.25);
	    radarAltitude = owner.registerFSXVariable("RADIO HEIGHT");
	}
	
	public void update() {
		repaint();
	}
	
	@Override
	protected void drawScale( final double r, final Point2D cp, Graphics2D g2) {
		setMinValue( 0 );
	    setMaxValue( 5 );
	    setMajorTickStep(1);
	    setMinorTickStep(.1);
	    setAngleStart(270);
	    setAngleExtend(180);
	    super.drawScale( r, cp, g2 );

		setMinValue( 5 );
	    setMaxValue( 30 );
	    setMajorTickStep(5);
	    setMinorTickStep(1);
	    setAngleStart(180);
	    setAngleExtend(90);
	    super.drawScale( r, cp, g2);

	}
	
	@Override
    protected void drawNeedle( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
		value = radarAltitude.getDoubleValue();
		if( value < 500 ) {
			setMinValue( 0 );
		    setMaxValue( 5 );
		    setMajorTickStep(1);
		    setMinorTickStep(.1);
		    setAngleStart(270);
		    setAngleExtend(180);
		}
		else {
			value = value > 3000 ? 3000 : value;
			setMinValue( 5 );
		    setMaxValue( 30 );
		    setMajorTickStep(5);
		    setMinorTickStep(1);
		    setAngleStart(180);
		    setAngleExtend(90);
			
		}
		setValue( value /= 100 );
	    super.drawNeedle(r, needleR, cp, g2);
	}
	
	@Override
	protected void drawValue( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
	
	}
}
