package org.gaugebook.gauges.aviation.simple;

import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.ScrollPane;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.table.TableModel;

import org.gaugebook.AbstractGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;

import flightsim.simconnect.SimConnect;
import flightsim.simconnect.SimConnectDataType;
import flightsim.simconnect.recv.DispatcherTask;
import flightsim.simconnect.recv.ExceptionHandler;
import flightsim.simconnect.recv.OpenHandler;
import flightsim.simconnect.recv.RecvException;
import flightsim.simconnect.recv.RecvOpen;
import flightsim.simconnect.wrappers.DataWrapper;

/**
 * SyncGauge
 * 
 * EXPERIMENTAL: Gauge to sync another FSX machine with the machine GaugeBook is listening to.
 * 
 * @author Michael Nagler
 */
public class SyncGauge extends AbstractGauge {
	
	private int EVENT_ID_SYNC = 1;
	
	private GaugeBook owner = null;
	private String syncIP = "192.168.178.28";
	private int syncPort = 9017;
	private Vector<FSXVariable> fsxvars = new Vector<FSXVariable>();
	
	JTextArea output;
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
		this.owner = owner;

		fsxvars.add( owner.registerFSXVariable("VELOCITY BODY Z"));
		fsxvars.add( owner.registerFSXVariable("VELOCITY BODY X"));
		fsxvars.add( owner.registerFSXVariable("VELOCITY BODY Y"));
		fsxvars.add( owner.registerFSXVariable("VELOCITY WORLD Z"));
		fsxvars.add( owner.registerFSXVariable("VELOCITY WORLD X"));
		fsxvars.add( owner.registerFSXVariable("VELOCITY WORLD Y"));
		fsxvars.add( owner.registerFSXVariable("ACCELERATION WORLD X"));
		fsxvars.add( owner.registerFSXVariable("ACCELERATION WORLD Y"));
		fsxvars.add( owner.registerFSXVariable("ACCELERATION WORLD Z"));
		fsxvars.add( owner.registerFSXVariable("ACCELERATION BODY X"));
		fsxvars.add( owner.registerFSXVariable("ACCELERATION BODY Y"));
		fsxvars.add( owner.registerFSXVariable("ACCELERATION BODY Z"));
		fsxvars.add( owner.registerFSXVariable("ROTATION VELOCITY BODY X"));
		fsxvars.add( owner.registerFSXVariable("ROTATION VELOCITY BODY Y"));
		fsxvars.add( owner.registerFSXVariable("ROTATION VELOCITY BODY Z"));
		fsxvars.add( owner.registerFSXVariable("PLANE ALT ABOVE GROUND"));
		fsxvars.add( owner.registerFSXVariable("PLANE LATITUDE"));
		fsxvars.add( owner.registerFSXVariable("PLANE LONGITUDE"));
		fsxvars.add( owner.registerFSXVariable("PLANE PITCH DEGREES"));
		fsxvars.add( owner.registerFSXVariable("PLANE BANK DEGREES"));
		fsxvars.add( owner.registerFSXVariable("PLANE ALTITUDE"));

		output = new JTextArea();
		setLayout(new GridLayout(1,1));
		add( new JScrollPane().add(output));
		
		addKeyListener(owner);

		try {
			sc = new SimConnect("SetVariable", syncIP, syncPort);
			dt = new DispatcherTask(sc);
			for( int i = 0; i < fsxvars.size(); i++ ) {
				FSXVariable var = fsxvars.get(i);
				sc.addToDataDefinition(EVENT_ID_SYNC, var.getName(), var.getUnit(), var.getType());
			}
			dt.addOpenHandler(new OpenHandler(){
				public void handleOpen(SimConnect sender, RecvOpen e) {
					System.out.println("SyncGauge Connected to " + e.getApplicationName()+"\n");
					connected = true;
				}
			});
			dt.addExceptionHandler(new ExceptionHandler(){
				public void handleException(SimConnect sender, RecvException e) {
					System.out.println("Exception (" + e.getException() +") packet " + e.getSendID()+"\n");
				}
			});

			new Thread(dt).start();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
	}
	
	private long lastFrame = 0;
	private SimConnect sc = null; 
	DispatcherTask dt;
	private int count = 0;
	private boolean connected = false;
	private boolean dispatch = false;
	public void update() {
		if( System.currentTimeMillis() - lastFrame > 100 ) {
			try {
				lastFrame = System.currentTimeMillis();
				if( connected ) {
					System.out.println("update event "+ EVENT_ID_SYNC + " time: "+ lastFrame );
					DataWrapper dw = new DataWrapper(fsxvars.size()*8);
					for( int i = 0; i < fsxvars.size(); i++ ) {
						FSXVariable var = fsxvars.get(i);
						dw.putFloat64(var.getDoubleValue());
					}
					sc.setDataOnSimObject(EVENT_ID_SYNC, SimConnect.OBJECT_ID_USER, false, 1, dw);
					sc.callDispatch(dt);
				}

			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
    protected void paintComponent( Graphics g ) {
    	output.repaint();
	}
    
	public void repaint() {
	}
}

