package org.gaugebook.gauges.aviation.simple;

import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;

/**
 * Temperature and Volts Gauge
 * 
 * @author Nagel
 */
public class TempVolts extends AbstractClockGauge {
	
	private FSXVariable varTemp, varVolts;

	private double t = 0, ct = 0, v = 0, cv = 0;
	private final int angleStartLeft = 315, angleExtendLeft = 90, angleStartRight = -225, angleExtendRight = 90;
	private final int minValueLeft = -20, maxValueLeft = 60, minValueRight = 0, maxValueRight = 30;
	private final int minorTickStepLeft = 10, majorTickStepLeft = 20, minorTickStepRight = 5, majorTickStepRight = 10;
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
		this.setMinValue( 0 );
        this.setMaxValue( 30 );
        this.setText("");
        this.setDecPlaceCount(0);
        this.setMajorTickStep(5);
        this.setMinorTickStep(1);
        this.setAngleExtend(angleExtendLeft);
        this.setAngleStart(angleStartLeft);
        this.setValue(0);
        this.setShowValue(false);
	    super.setTextPosition(1.0, 0.2);
	    super.setTextHeight( 1.4 ); 

	    varTemp  = owner.registerFSXVariable("AMBIENT TEMPERATURE");
	    varVolts = owner.registerFSXVariable("ELECTRICAL BATTERY VOLTAGE");
    }
	
	public void update() {

		t = varTemp.getDoubleValue();
		v = varVolts.getDoubleValue();
		if( Math.round(ct) != Math.round(t) || Math.round(cv) != Math.round(v)) {
			setValue(ct = t );
			cv = v;
		}
		
		repaint();
		
	}
	
    protected void drawScale( final double r, final Point2D cp, Graphics2D g2 ) {
    	double f = 1;
    	Point2D left = new Point2D.Double(cp.getX() - r * f, cp.getY());
    	Point2D right= new Point2D.Double(cp.getX() + r * f, cp.getY());
        setAngleExtend(angleExtendRight);
        setAngleStart(angleStartRight);
        setMinorTickStep(minorTickStepRight);
        setMajorTickStep(majorTickStepRight);
        setMinValue(minValueRight);
        setMaxValue(maxValueRight);
    	super.drawScale(r, right, g2, Direction.CLOCKWISE);
        setAngleExtend(angleExtendLeft);
        setAngleStart(angleStartLeft);
        setMinorTickStep(minorTickStepLeft);
        setMajorTickStep(majorTickStepLeft);
        setMinValue(minValueLeft);
        setMaxValue(maxValueLeft);
    	super.drawScale(r, left, g2, Direction.ANTICLOCKWISE);
    	
    }

    protected void drawNeedle( final double r, double needleR, final Point2D cp, Graphics2D g2 ) {
    	Shape oldClip = g2.getClip();
    	double cf = .87; // clip factor
    	Ellipse2D.Double ell = new Ellipse2D.Double( cp.getX() - r * cf, cp.getY() - r * cf, 2 * r * cf, 2 * r * cf );
		g2.setClip(ell);

    	double f = 1;
    	Point2D left = new Point2D.Double(cp.getX() - r * f, cp.getY());
    	Point2D right= new Point2D.Double(cp.getX() + r * f, cp.getY());
        setAngleExtend(angleExtendRight);
        setAngleStart(angleStartRight);
        setMinorTickStep(minorTickStepRight);
        setMajorTickStep(majorTickStepRight);
        setMinValue(minValueRight);
        setMaxValue(maxValueRight);
    	super.drawNeedle(r, r * .7, right, g2, cv);
        setAngleExtend(angleExtendLeft);
        setAngleStart(angleStartLeft);
        setMinorTickStep(minorTickStepLeft);
        setMajorTickStep(majorTickStepLeft);
        setMinValue(minValueLeft);
        setMaxValue(maxValueLeft);
    	super.drawNeedle(r, r * .7, left, g2, ct, Direction.ANTICLOCKWISE );
    	
		g2.setClip(oldClip);
    }
    
    protected void drawText( final double r, final Point2D cp, Graphics2D g2 ) {
    	super.drawText(r, cp, g2);
    	super.drawText(r, cp, g2, new Point2D.Double(-.65, .05), 1.1, "Temp");
    	super.drawText(r, cp, g2, new Point2D.Double(-.65, .15), 1.0, "C");
    	super.drawText(r, cp, g2, new Point2D.Double( .65, .05), 1.1, "Batt");
    	super.drawText(r, cp, g2, new Point2D.Double( .65, .15), 1.0, "Volts");
    	
    }
}
