package org.gaugebook.gauges.aviation.simple;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Point2D;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;

/**
 * TurbRotorRPM
 * 
 * @author Michael Nagler
 */
public class TurbRotorRPM extends AbstractClockGauge {
	
	private FSXVariable torqRPM, rotorRPM;
	private boolean initdone = false;
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
		this.setMinValue( 0 );
        this.setMaxValue( 120 );
        this.setText("PERCENT RPM");
        this.setDecPlaceCount(0);
        this.setMajorTickStep(10);
        this.setMinorTickStep(2);
        this.setAngleExtend(330);
        this.setAngleStart(270);
        this.setValue(0);
        this.setShowValue(false);
	    super.setTextPosition(0, -.25);
	    // Torque. Returns main rotor torque for Bell helicopter, or the indexed rotor torque of other helicopters. / Percent scalar 16K (Ft/lbs * 16384) 
	    torqRPM = owner.registerFSXVariable("ENG TORQUE PERCENT:1"); // GENERAL ENG PCT MAX RPM:1");
	 	//  	Rotor rpm. Returns main rotor rpm for Bell helicopter, or the indexed rotor rpm of other helicopters./ Percent scalar 16K (Max rpm * 16384) 
	    rotorRPM = owner.registerFSXVariable("ENG ROTOR RPM:1");
	}
	
	protected void drawNeedle( final double r, final double needleR, final Point2D cp, Graphics2D g2 ) {
		super.drawNeedle(r, needleR, cp, g2, torqRPM.getDoubleValue() ); // 16384 * 100 );
		//super.drawNeedle(r, needleR, cp, g2, rotorRPM.getDoubleValue() ); //  / 16384 * 100 );
	}
		   
	protected void drawScale( final double r, final Point2D cp, Graphics2D g2 ) {
    	super.drawArc( 20, 26, new Color(0,140,0), r, .85, .9, cp, g2 );
    	super.drawScale(r, cp, g2);
    	super.drawMarker(26, new Color(160,0,0), 3, r, .8, .9, cp, g2);
    }
}
