package org.gaugebook.gauges.aviation.simple;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Point2D;

import org.gaugebook.AbstractClockGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;


public class VACAMP extends AbstractClockGauge {
	
	private FSXVariable varVAC, varAMP;

	private double vac = 0, cvac = 0, amp = 0, camp = 0;
	private final int angleStartLeft = 315, angleExtendLeft = 90, angleStartRight = -225, angleExtendRight = 90;
	private final int minValueLeft = 3, maxValueLeft = 7, minValueRight = -60, maxValueRight = 60;
	private final int minorTickStepLeft = 1, majorTickStepLeft = 1, minorTickStepRight = 15, majorTickStepRight = 30;
	
	public void init( GaugeBook owner, String title ) {
		super.init( owner, title );
		this.setMinValue( 0 );
        this.setMaxValue( 30 );
        this.setText("");
        this.setDecPlaceCount(0);
        this.setMajorTickStep(5);
        this.setMinorTickStep(1);
        this.setAngleExtend(angleExtendLeft);
        this.setAngleStart(angleStartLeft);
        this.setValue(0);
        this.setShowValue(false);
	    super.setTextPosition(0, -.6);
	    super.setTextHeight( 1.4 ); 

	    varVAC = owner.registerFSXVariable("SUCTION PRESSURE");
	    varAMP = owner.registerFSXVariable("ELECTRICAL TOTAL LOAD AMPS");
    }
	
	public void update() {
		vac = varVAC.getDoubleValue();
		amp = varAMP.getDoubleValue();
		if( Math.round(cvac) != Math.round(vac) || Math.round(camp) != Math.round(amp)) {
			setValue(cvac = vac );
			camp = amp;
		}
		repaint();
	}
	
    protected void drawScale( final double r, final Point2D cp, Graphics2D g2 ) {
    	double f = 1;
    	Point2D left = new Point2D.Double(cp.getX() - r * f, cp.getY());
    	Point2D right= new Point2D.Double(cp.getX() + r * f, cp.getY());
        setAngleExtend(angleExtendRight);
        setAngleStart(angleStartRight);
        setMinorTickStep(minorTickStepRight);
        setMajorTickStep(majorTickStepRight);
        setMinValue(minValueRight);
        setMaxValue(maxValueRight);
    	super.drawScale(r, right, g2, Direction.CLOCKWISE);
        setAngleExtend(angleExtendLeft);
        setAngleStart(angleStartLeft);
        setMinorTickStep(minorTickStepLeft);
        setMajorTickStep(majorTickStepLeft);
        setMinValue(minValueLeft);
        setMaxValue(maxValueLeft);
    	super.drawArc(  4.5, 5.5, new Color(0,160,0), r, .87, .91, left, g2 );
    	super.drawScale(r, left, g2, Direction.ANTICLOCKWISE);
    	
    }

    protected void drawNeedle( final double r, double needleR, final Point2D cp, Graphics2D g2 ) {
    	Shape oldClip = g2.getClip();
    	double cf = .87; // clip factor
    	Ellipse2D.Double ell = new Ellipse2D.Double( cp.getX() - r * cf, cp.getY() - r * cf, 2 * r * cf, 2 * r * cf );
		g2.setClip(ell);

    	double f = 1;
    	Point2D left = new Point2D.Double(cp.getX() - r * f, cp.getY());
    	Point2D right= new Point2D.Double(cp.getX() + r * f, cp.getY());
        setAngleExtend(angleExtendRight);
        setAngleStart(angleStartRight);
        setMinorTickStep(minorTickStepRight);
        setMajorTickStep(majorTickStepRight);
        setMinValue(minValueRight);
        setMaxValue(maxValueRight);
        double v = camp;
        if( v < minValueRight || v > maxValueRight ) { 
            v = (value < minValueRight ? minValueRight : maxValueRight);
        }
    	super.drawNeedle(r, r * .7, right, g2, v ); // inHg
        setAngleExtend(angleExtendLeft);
        setAngleStart(angleStartLeft);
        setMinorTickStep(minorTickStepLeft);
        setMajorTickStep(majorTickStepLeft);
        setMinValue(minValueLeft);
        setMaxValue(maxValueLeft);
        v = cvac;
        if( v < minValueLeft || v > maxValueLeft ) { 
            v = (value < minValueLeft ? minValueLeft : maxValueLeft);
        }
    	super.drawNeedle(r, r * .7, left, g2, v, Direction.ANTICLOCKWISE );
    	
		g2.setClip(oldClip);
    }
    
    protected void drawText( final double r, final Point2D cp, Graphics2D g2 ) {
    	super.drawText(r, cp, g2);
    	super.drawText(r, cp, g2, new Point2D.Double( .65, .1), 1.1, "AMP");
    	super.drawText(r, cp, g2, new Point2D.Double(-.65, .05), 1.1, "VAC");
    	super.drawText(r, cp, g2, new Point2D.Double(-.65, .15), 1.0, "inHg");

    }
}
