package org.gaugebook.gauges.aviation.special;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.image.VolatileImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;

import org.gaugebook.AbstractGauge;
import org.gaugebook.GaugeBook;
import org.gaugebook.LED;
import org.gaugebook.math.Angle;

import com.sun.corba.se.impl.ior.ByteBuffer;
import com.sun.pdfview.PDFFile;
import com.sun.pdfview.PDFPage;
import com.sun.pdfview.PDFRenderer;

public class ChartViewerGauge extends AbstractGauge implements KeyListener {

	/*	
	 * FKeybar - beta solution
	 * 
	 * F1 - OPEN 	- open file dialog, select your chart
	 * F2 - PREV	- prev page
	 * F3 - NEXT	- next page
	 * F2 - WIDTH	- zoom to page width
	 * F3 - PAGE	- zoom to show full page
	 * F4 - ROTL	- rotate left
	 * F5 - ROTR	- rotate right
	 * F6 - ZOOM+	- zoom in
	 * F7 - ZOOM-	- zoom out
	 * 
	 */
//	private FKey[] fkeys = 
//		new FKey("OPEN", "NONE", )

	// beta code to be improved  
	private String[] fkeylabels = {"OPEN","PREV","NEXT","(FIT)","ROTL","ROTR","(ZOOM+)","(ZOOM-)","","","",""};
	private String[] fkeyevents = {null, null, null,  null, null, null,  null, null, null,  null, null, null };
	private boolean[] fkeyleds  = { false, false, false, false, false, false, false, false, false, false, false, false };
	public String[] getFkeylabels() { return fkeylabels; }
	public String[] getFkeyevents() { return fkeyevents; }
	public boolean[] getFkeyleds() { return fkeyleds; }

	
	/*
	 * FKeybar - smart solution (for a later version)
	 * 
	 * If no PDF is open, the manual.pdf is shown, the FKeybars have the following functions:
	 * 
	 * F1 = DEPT     - select departure airport
	 * F2 = DEST     - select destination airport
	 * F3 = CUSTOM   - select custom airport (fms input dialog)
	 * F4 = OPEN     - open file
	 * F5 = HISTORY          
	 * F6 =
	 * 
	 * If an airport is selected, the FKeybars change their functions to:
	 * F1 = INFO	 - select general information PDFs
	 * F2 = GROUND	 - select ground PDFs
	 * F3 = PARK	 - select parking PDFs
	 * F4 = STARS    - 
	 * F5 = APPR ILS 
	 * F6 = APPR NDB - 
	 * F7 = APPR RNAV- 
	 * F8 = DEPT     -
	 * F9 = VFR
	 * 
	 * If the type is selected the ChartViewer presents a list of available PDFs, e.g. if APPR ILS is selected for EDDF:
	 * F1 = 08L
	 * F2 = 08R
	 * F3 = 26L
	 * F4 = 26R
	 * 
	 *  
	 * F7 = WIDTH    - zoom to page width (default)  // these keys only show up when a PDF is open
	 * F8 = PAGE     - zoom to page fit
	 * F9 = ZOOM IN  - zoom in
	 * F10= ZOOM OUT - zoom out
	 * F11= ROTATE L - rotate left
	 * F12= ROTATE R - rotate right
	 * 
 	 * cursor keys: move left, right, up and down
 	 * 
	 */
	PDFRenderer renderer;
	PDFFile pdffile;
	PDFPage page;
	private int lastWidth = 0;
	private int lastHeight = 0;
	private int pagenum = 0;
	Angle rotateAngle = new Angle( 0 );
	boolean fileChanged = false;
	GaugeBook owner = null;
	
	private int dy = 0;
	private int index;
	
	public ChartViewerGauge() {
		super();
	}

	@Override
	public void init( GaugeBook owner, String title ) {
		this.owner = owner;
    	loadPDF( new File( owner.welcomePDFile ));
    	this.title = title;
	}
	
	protected void loadPDF( File file ) {
	    RandomAccessFile raf = null;
	    MappedByteBuffer buf = null;
		try {
			raf = new RandomAccessFile(file, "r");
		    FileChannel channel = raf.getChannel();
		    buf = channel.map(FileChannel.MapMode.READ_ONLY, 0, channel.size());
		    pdffile = new PDFFile(buf);
		    fileChanged = true;
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	protected void loadPDF( String name ) {
	    File file = new File("charts/"+ name);
	    loadPDF( file );
	}
	
	int lastPagenum = -1;
	int lastRotation = 0;
	int lastDy = 0;
	
    public void paint( Graphics g ) {
        int w = getWidth();
        int h = getHeight();

        GraphicsConfiguration gc = getGraphicsConfiguration();

    	if( pdffile == null ) return;

		// draw the first page to an image
    	if( lastWidth != w || lastHeight != h || lastDy != dy || fileChanged || pagenum != lastPagenum || rotateAngle.getDegrees() != lastRotation
    			|| vi == null || vi.validate(gc) == VolatileImage.IMAGE_INCOMPATIBLE ) {
    		fileChanged = false;
        	lastPagenum = pagenum;
        	lastDy = dy;
    		lastRotation = (int)rotateAngle.getDegrees();
		    page = pdffile.getPage(pagenum);

		    //get the width and height for the doc at the default zoom
		    AffineTransform at = new AffineTransform();
			at.scale( w / page.getWidth(), h / page.getHeight() );
		    page.addXform(at);

		    int pw = (int)page.getBBox().getWidth();
		    int ph = (int)page.getBBox().getHeight();
		    
		    if( pw < w ) {
		    	pw = w;
		    }
	        int k = Math.max( pw, ph ); // kante

    		do {
	    		gc = getGraphicsConfiguration();
	        	if( vi == null || vi.validate(gc) == VolatileImage.IMAGE_INCOMPATIBLE ) {
	        		vi = gc.createCompatibleVolatileImage( k, k );
	        	}
	
	        	Graphics2D g2 = vi.createGraphics();
	        	g2.setColor( Color.white );
	        	g2.fillRect( 0, 0, k, k );
	        	
		
//	System.out.println("Pagenum: "+page.getPageNumber() +" Rotation: "+ page.getRotation() +" RotAngle="+ rotateAngle.getRadians() 
			//+" PageRotation: "+ page.getRotation() +" w="+w+" h="+h+" pw="+pw+" ph="+ph);
	
			    Rectangle rect = new Rectangle(0,0,w,ph);
			    Rectangle clip = new Rectangle(0,0,w,h);
	
		    	try {
				    page.waitForFinish();
				} catch (InterruptedException e) {
				}
	
			    //generate the image
			    Image img = page.getImage(
			            rect.width, rect.height, //width & height
			            clip, // clip rect
			            null, // null for the ImageObserver
			            true, // fill background with white
			            true  // block until drawing is done
			            );
			    
				switch( (int)rotateAngle.getDegrees() ) {
				case 90:
				    at = new AffineTransform();
					at.rotate( rotateAngle.getRadians(), 0, 0 ); //, page.getWidth()/2, page.getHeight()/2 ); // rotateAngle.getDegrees() );
					at.translate( 0, -w );
					at.scale( h / page.getWidth(), w / page.getHeight() );
					g2.setTransform(at);
					g2.drawImage(img, 0, 0, w, h, 0, dy, w, h + dy, this);
					break;
				case 180:
				    at = new AffineTransform();
					at.rotate( rotateAngle.getRadians(), 0, 0 ); //, page.getWidth()/2, page.getHeight()/2 ); // rotateAngle.getDegrees() );
					at.translate( -w, -h );
					g2.setTransform(at);
					g2.drawImage(img, 0, 0, w, h, 0, dy, w, h + dy, this);
					break;
				case 270:
				    at = new AffineTransform();
					at.rotate( rotateAngle.getRadians(), 0, 0 ); //, page.getWidth()/2, page.getHeight()/2 ); // rotateAngle.getDegrees() );
					at.translate( -h, 0 );
					at.scale( h / page.getWidth(), w / page.getHeight() );
					g2.setTransform(at);
					g2.drawImage(img, 0, 0, w, h, 0, dy, w, h + dy, this);
					break;
				default:
					g2.drawImage(img, 0, 0, w, h, 0, dy, w, h + dy, this);
				}

				g2.dispose();
    		} while( vi.contentsLost() );
    	}
    	g.drawImage( vi, 0, 0, this );
	}

    public void update() {
    	repaint();
    }
    
	@Override
	public void keyPressed(KeyEvent e) {
		if( e.isAltDown() ) return;
		
		switch( e.getKeyCode() ) {
		case KeyEvent.VK_F1:
			// show file dialog
			JFileChooser fileDialog = new JFileChooser();
			fileDialog.setCurrentDirectory(new File(owner.getPdfPath()));
			fileDialog.setFileFilter(new FileFilter() {
				@Override
				public boolean accept(File f) {
					// TODO Auto-generated method stub
					String ext = null;
			        String s = f.getName();
			        int i = s.lastIndexOf('.');

			        if (i > 0 &&  i < s.length() - 1) {
			            ext = s.substring(i+1).toLowerCase();
			        }
			        return f.isDirectory() || ext != null && ext.equals("pdf");
				}
	            public String getDescription() {
	                return "PDF Charts (*.pdf)";
	            }
			});
			// fileDialog.setFileSelectionMode(JFileChooser.FILES_ONLY);
			fileDialog.setAcceptAllFileFilterUsed(false);
			// load pdf
			int returnVal = fileDialog.showOpenDialog(owner);
   			if (returnVal == JFileChooser.APPROVE_OPTION) {
   	            File file = fileDialog.getSelectedFile();
	       	 		loadPDF(file);
	       	}
			e.consume();
			break;
		case KeyEvent.VK_F2: // prev page
			if( pdffile != null && pdffile.getNumPages() > 1 ) {
				if( pagenum >= 0 ) pagenum--; else pagenum = pdffile.getNumPages() - 1;
			}
			e.consume();
			break;
		case KeyEvent.VK_F3: // next page
			if( pdffile != null && pdffile.getNumPages() > 1 ) {
				if( pagenum < pdffile.getNumPages() ) pagenum++; else pagenum = 0;
			}
			e.consume();
			break;
		case KeyEvent.VK_F4:
			 // fit page width
			e.consume();
			break;
		case KeyEvent.VK_F5:
			 // rotate left
			rotateAngle = new Angle( rotateAngle.getDegrees() - 90 );
			dy = 0;
			e.consume();
			break;
		case KeyEvent.VK_F6:
			 // rotate right
			rotateAngle = new Angle( rotateAngle.getDegrees() + 90 );
			dy = 0;
			e.consume();
			break;
		case KeyEvent.VK_F7:
			 // zoom out
			e.consume();
			break;
		case KeyEvent.VK_F8:
			 // zoom in
			e.consume();
			break;
		case KeyEvent.VK_F9:
			 // zoom out
			e.consume();
			break;
		case KeyEvent.VK_F10:
			e.consume();
			break;
		case KeyEvent.VK_F11:
			e.consume();
			break;
		case KeyEvent.VK_F12:
			e.consume();
			break;
		case KeyEvent.VK_PAGE_DOWN:
			if( (int)rotateAngle.getDegrees() == 0 && dy < page.getBBox().getHeight() - getHeight() ) {
				dy += 40;
			}
			e.consume();
			break;
		case KeyEvent.VK_PAGE_UP:
			if( (int)rotateAngle.getDegrees() == 0 && dy >= 40 ) {
				dy -= 40;
			};
			e.consume();
			break;
		default:
		}
	}

	@Override
	public void keyReleased(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
		// TODO Auto-generated method stub
		
	}

}
