package org.gaugebook.gauges.aviation.special;

import java.awt.Graphics;
import java.awt.GridLayout;
import java.util.Vector;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import org.gaugebook.AbstractGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;

import flightsim.simconnect.SimConnectDataType;

public class DebugGauge extends AbstractGauge {
	
	GaugeBook owner = null;
	JTable table;
	TableModel dataModel;
	JScrollPane scrollpane;

	private Vector<FSXVariable> vars = new Vector<FSXVariable>();

	private static String[] surfaceTypes = { "Concrete", "Grass", "Water", "Grass_bumpy", "Asphalt", "Short_grass",
		"Long_grass", "Hard_turf", "Snow", "Ice", "Urban",
		"Forest", "Dirt", "Coral", "Gravel", "Oil_treated",
		"Steel_mats", "Bituminus", "Brick", "Macadam", "Planks",
		"Sand", "Shale", "Tarmac", "Wright_flyer_track"		
	};

	public void init ( GaugeBook owner, String title ) {
		super.init( owner, title );
		this.owner = owner;
		
		vars.add( owner.registerFSXVariable("SURFACE TYPE"));
		vars.add( owner.registerFSXVariable("TITLE"));
		vars.add( owner.registerFSXVariable("ATC TYPE"));
		vars.add( owner.registerFSXVariable("ATC MODEL"));
		vars.add( owner.registerFSXVariable("ATC ID"));
		vars.add( owner.registerFSXVariable("ATC AIRLINE"));
		int i = 0;
		while( hasProperty("simvar"+i)) {
			vars.add( owner.registerFSXVariable((String)getProperty("simvar"+i)));
			i++;
		}
		
		addKeyListener(owner);
		initdone = true;
    }
	
	public void update() {
		if( dataModel == null ) {
			vars = owner.getFSXVariables();
			dataModel = new AbstractTableModel() {
				public int getColumnCount() { return 4; }
				public int getRowCount() { return vars.size();}
				public Object getValueAt(int row, int col) { 
					FSXVariable var = vars.get(row);
					Object result = null;
					switch( col ) {
					case 0: 
						result = new String( var.getName() ); 
						break;
					case 1: 
						if( var.getName() == "SURFACE TYPE" ) {
							result = new String( (surfaceTypes[(int)Math.round(var.getDoubleValue())] ));
						} else if( var.getType() == SimConnectDataType.STRING256 ){
							result = new String( var.getStringValue() );
						} else if( var.getType() == SimConnectDataType.STRING32 ){
							result = new String( var.getStringValue() );
						} else {
							result = new Double( var.getDoubleValue() );
						}
						break;
					case 2: result = new String( var.getUnit() ); break;
					case 3: result = new String( var.getDescription() ); break;
					default: result = ""; 
					}
					return result;
				}
			};
			table = new JTable(dataModel);
			table.addKeyListener(owner);
			scrollpane = new JScrollPane(table);
			setLayout(new GridLayout(1,1));
			add(scrollpane);
		}
		
		for( int i = 0; i < vars.size(); i++ ) {
			FSXVariable var = vars.elementAt(i);
			if( var.getType() == SimConnectDataType.STRING256 ){
				dataModel.setValueAt((Object)vars.get(i).getStringValue(), i, 1);
			} else if( var.getType() == SimConnectDataType.STRING32 ){
				dataModel.setValueAt((Object)vars.get(i).getStringValue(), i, 1);
			} else {
				dataModel.setValueAt((Object)vars.get(i).getDoubleValue(), i, 1);
			}
		}
		
		repaint();
	}
}
