package org.gaugebook.gauges.aviation.special;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.text.NumberFormat;
import java.util.Timer;
import java.util.TimerTask;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.border.TitledBorder;

import org.gaugebook.AbstractGauge;
import org.gaugebook.FSXVariable;
import org.gaugebook.GaugeBook;

import com.jcraft.jorbis.JOrbisStreamPlayer;


public class SimpleRadioPanel extends AbstractGauge {

	private static final long serialVersionUID = 1L;
	
	private FSXVariable varCom1Xmit, varCom2Xmit;
	private FSXVariable varNav1Xmit, varNav2Xmit;
	private FSXVariable varAdf1Xmit;
	private FSXVariable varComBoth;
	private FSXVariable varCom1Freq, varCom2Freq;
	private FSXVariable varCom1Stby, varCom2Stby;
	private FSXVariable varNav1Freq, varNav2Freq;
	private FSXVariable varNav1Stby, varNav2Stby;
	private FSXVariable varAdfFreq ;
	private FSXVariable varXpdrCode;
	private FSXVariable varZuluTime;
	private FSXVariable dmeXmit, markerXmit;
	private FSXVariable varAviMaster;
	private FSXVariable hasDME1, hasDME2, navDME1, navDME2;
	private FSXVariable atcType, atcModel, atcID;
	
	private double com1Xmit, com2Xmit, nav1Xmit, nav2Xmit, adf1Xmit, comBoth, com1Freq, com2Freq, com1Stby, com2Stby,
				nav1Freq, nav2Freq, nav1Stby, nav2Stby, adfFreq, xpdrCode, zuluTime;
	private boolean aviMaster = false;
	
	private GaugeBook owner;
	private String title;
	private int frames = 0;

    private JOrbisStreamPlayer player = new JOrbisStreamPlayer();
    private float radioPlaying = -1; // radio freq , -1 if not playing 
	
	private static NumberFormat freqNF = NumberFormat.getInstance();
	private static NumberFormat clkNF  = NumberFormat.getIntegerInstance();
	private static NumberFormat dmeNF  = NumberFormat.getInstance();
	
	JLabel lblCom1Freq = new JLabel("      ");
	JLabel lblCom1Swap = new JLabel("\u2194");
	JLabel lblCom1Stby = new JLabel("      ");

	JLabel lblCom2Freq = new JLabel("      ");
	JLabel lblCom2Swap = new JLabel("\u2194");
	JLabel lblCom2Stby = new JLabel("      ");

	JLabel lblNav1Freq = new JLabel("      ");
	JLabel lblNav1Swap = new JLabel("\u2194");
	JLabel lblNav1Stby = new JLabel("      ");

	JLabel lblNav2Freq = new JLabel("      ");
	JLabel lblNav2Swap = new JLabel("\u2194");
	JLabel lblNav2Stby = new JLabel("      ");

	JLabel lblAdfFreq1  = new JLabel("      ");
	JLabel lblXpdrCode = new JLabel("      ");
	
	JLabel lblDME1     = new JLabel("      ");
	JLabel lblDME2     = new JLabel("      ");
	
	JLabel lblFPS      = new JLabel("");
	JLabel lblGBFPS	   = new JLabel("");
	JLabel lblZuluTime = new JLabel("");
	
	JPanel pnlCom1 = new JPanel( new BorderLayout() );
	JPanel pnlCom2 = new JPanel( new BorderLayout() );
	JPanel pnlNav1 = new JPanel( new BorderLayout() );
	JPanel pnlNav2 = new JPanel( new BorderLayout() );
	JPanel pnlDME = new JPanel( new BorderLayout() );
	JPanel pnlAdf1 = new JPanel( new BorderLayout() );
	
	TitledBorder tbCom1 = BorderFactory.createTitledBorder("COM1 [_]");
	TitledBorder tbCom2 = BorderFactory.createTitledBorder("COM2 [_]");
	TitledBorder tbNav1 = BorderFactory.createTitledBorder("NAV1 [_]");
	TitledBorder tbNav2 = BorderFactory.createTitledBorder("NAV2 [_]");
	TitledBorder tbDME  = BorderFactory.createTitledBorder("DME [_] / MRKR [_]");
	TitledBorder tbAdf1 = BorderFactory.createTitledBorder("ADF [_] / XPDR");
	
	private int frameCounter;
	private String gbTitle;
	
	public SimpleRadioPanel(GaugeBook owner, String title) {
		this.owner = owner;
		this.title = title;
		gbTitle = owner.getTitle();
		setLayout( new GridLayout(1,7));
        setBackground(Color.DARK_GRAY);
        
    	varCom1Xmit = owner.registerFSXVariable("COM TRANSMIT:1");
    	varCom2Xmit = owner.registerFSXVariable("COM TRANSMIT:2");
    	varNav1Xmit = owner.registerFSXVariable("NAV SOUND:1");
    	varNav2Xmit = owner.registerFSXVariable("NAV SOUND:2");
    	varAdf1Xmit = owner.registerFSXVariable("ADF SOUND:1");
    	varComBoth  = owner.registerFSXVariable("COM RECIEVE ALL");
    	varCom1Freq = owner.registerFSXVariable("COM ACTIVE FREQUENCY:1");
    	varCom2Freq = owner.registerFSXVariable("COM ACTIVE FREQUENCY:2");
    	varCom1Stby = owner.registerFSXVariable("COM STANDBY FREQUENCY:1");
    	varCom2Stby = owner.registerFSXVariable("COM STANDBY FREQUENCY:2");
    	varNav1Freq = owner.registerFSXVariable("NAV ACTIVE FREQUENCY:1");
    	varNav2Freq = owner.registerFSXVariable("NAV ACTIVE FREQUENCY:2");
    	varNav1Stby = owner.registerFSXVariable("NAV STANDBY FREQUENCY:1");
    	varNav2Stby = owner.registerFSXVariable("NAV STANDBY FREQUENCY:2");
    	varAdfFreq  = owner.registerFSXVariable("ADF ACTIVE FREQUENCY:1");
    	varXpdrCode = owner.registerFSXVariable("TRANSPONDER CODE:1");
    	varZuluTime = owner.registerFSXVariable("ZULU TIME");
    	varAviMaster = owner.registerFSXVariable("AVIONICS MASTER SWITCH");
    	dmeXmit = owner.registerFSXVariable("DME SOUND");
    	markerXmit = owner.registerFSXVariable("MARKER SOUND");

		hasDME1 = owner.registerFSXVariable("NAV HAS DME:1");
		hasDME2 = owner.registerFSXVariable("NAV HAS DME:2");
		navDME1 = owner.registerFSXVariable("NAV DME:1");
		navDME2 = owner.registerFSXVariable("NAV DME:2");

		atcType = owner.registerFSXVariable("ATC TYPE");
		atcModel = owner.registerFSXVariable("ATC MODEL");
		atcID = owner.registerFSXVariable("ATC ID");

		JPanel bp; // build panel
		
		// COM1 Panel
		pnlCom1.setBackground(Color.DARK_GRAY);
		tbCom1.setTitleColor(Color.WHITE);			
		tbCom1.setTitleFont(tbCom1.getTitleFont().deriveFont(Math.round(tbCom1.getTitleFont().getSize() * .9)));
        pnlCom1.setBorder( BorderFactory.createCompoundBorder( tbCom1, BorderFactory.createEmptyBorder(-4,4,0,4))); 
		lblCom1Freq.setForeground(Color.RED);
		lblCom1Swap.setForeground(Color.WHITE);
		lblCom1Stby.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblCom1Freq); pnlCom1.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.DARK_GRAY); bp.add(lblCom1Swap); pnlCom1.add(bp, BorderLayout.CENTER);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblCom1Stby); pnlCom1.add(bp, BorderLayout.EAST);

		// COM2 Panel
		pnlCom2.setBackground(Color.DARK_GRAY);
		tbCom2.setTitleColor(Color.WHITE);
		tbCom2.setTitleFont(tbCom2.getTitleFont().deriveFont(Math.round(tbCom2.getTitleFont().getSize() * .9)));
        pnlCom2.setBorder( BorderFactory.createCompoundBorder( tbCom2, BorderFactory.createEmptyBorder(-4,4,0,4))); 
		lblCom2Freq.setForeground(Color.RED);
		lblCom2Swap.setForeground(Color.WHITE);
		lblCom2Stby.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblCom2Freq); pnlCom2.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.DARK_GRAY); bp.add(lblCom2Swap); pnlCom2.add(bp, BorderLayout.CENTER);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblCom2Stby); pnlCom2.add(bp, BorderLayout.EAST);

		// NAV1 Panel
		pnlNav1.setBackground(Color.DARK_GRAY);
		tbNav1.setTitleColor(Color.WHITE);
		tbNav1.setTitleFont(tbNav1.getTitleFont().deriveFont(Math.round(tbNav1.getTitleFont().getSize() * .9)));
        pnlNav1.setBorder( BorderFactory.createCompoundBorder( tbNav1, BorderFactory.createEmptyBorder(-4,4,0,4))); 
		lblNav1Freq.setForeground(Color.RED);
		lblNav1Swap.setForeground(Color.WHITE);
		lblNav1Stby.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblNav1Freq); pnlNav1.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.DARK_GRAY); bp.add(lblNav1Swap); pnlNav1.add(bp, BorderLayout.CENTER);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblNav1Stby); pnlNav1.add(bp, BorderLayout.EAST);
		
		// NAV2 Panel
		pnlNav2.setBackground(Color.DARK_GRAY);
		tbNav2.setTitleColor(Color.WHITE);
		tbNav2.setTitleFont(tbNav2.getTitleFont().deriveFont(Math.round(tbNav2.getTitleFont().getSize() * .9)));
        pnlNav2.setBorder( BorderFactory.createCompoundBorder( tbNav2, BorderFactory.createEmptyBorder(-4,4,0,4))); 
		lblNav2Freq.setForeground(Color.RED);
		lblNav2Swap.setForeground(Color.WHITE);
		lblNav2Stby.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblNav2Freq); pnlNav2.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.DARK_GRAY); bp.add(lblNav2Swap); pnlNav2.add(bp, BorderLayout.CENTER);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblNav2Stby); pnlNav2.add(bp, BorderLayout.EAST);

		// DME Panel
		pnlDME.setBackground(Color.DARK_GRAY);
		tbDME.setTitleColor(Color.WHITE);
		tbDME.setTitleFont(tbDME.getTitleFont().deriveFont(Math.round(tbDME.getTitleFont().getSize() * .9)));
        pnlDME.setBorder( BorderFactory.createCompoundBorder( tbDME, BorderFactory.createEmptyBorder(-4,4,0,4) )); 
		lblDME1.setForeground(Color.RED);
		lblDME2.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblDME1); pnlDME.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.DARK_GRAY); pnlDME.add(bp, BorderLayout.CENTER);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblDME2); pnlDME.add(bp, BorderLayout.EAST);

		// XPDR / ADF Panel
		pnlAdf1.setBackground(Color.DARK_GRAY);
		tbAdf1.setTitleColor(Color.WHITE);
		tbAdf1.setTitleFont(tbAdf1.getTitleFont().deriveFont(Math.round(tbAdf1.getTitleFont().getSize() * .9)));
        pnlAdf1.setBorder( BorderFactory.createCompoundBorder( tbAdf1, BorderFactory.createEmptyBorder(-4,4,0,4) )); 
		lblAdfFreq1.setForeground(Color.RED);
		lblXpdrCode.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblAdfFreq1); pnlAdf1.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.DARK_GRAY); pnlAdf1.add(bp, BorderLayout.CENTER);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblXpdrCode); pnlAdf1.add(bp, BorderLayout.EAST);
		
/*		// FPS Panel
		JPanel pnlFPS = new JPanel(new GridLayout());
		pnlFPS.setBackground(Color.DARK_GRAY);
		tb = BorderFactory.createTitledBorder("FPS");
		tb.setTitleColor(Color.WHITE);
        pnlFPS.setBorder( BorderFactory.createCompoundBorder( tb, BorderFactory.createEmptyBorder(0,4,0,4) )); 
		lblFPS.setForeground(Color.RED);
		lblGBFPS.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblFPS); pnlFPS.add(bp, BorderLayout.WEST);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblGBFPS); pnlFPS.add(bp, BorderLayout.WEST);

		// Time Panel
		JPanel pnlTime = new JPanel(new BorderLayout());
		pnlTime.setBackground(Color.DARK_GRAY);
		tb = BorderFactory.createTitledBorder("Clock");
		tb.setTitleColor(Color.WHITE);
        pnlTime.setBorder( BorderFactory.createCompoundBorder( tb, BorderFactory.createEmptyBorder(-4,4,0,4) )); 
        lblZuluTime.setForeground(Color.RED);
		bp = new JPanel(); bp.setBackground(Color.BLACK); bp.add(lblZuluTime); pnlTime.add(bp, BorderLayout.WEST);
*/

		lblCom1Freq.setName("COM1FREQ");
		lblCom1Swap.setName("COM1SWAP");
		lblCom1Stby.setName("COM1STBY");
		lblCom2Freq.setName("COM2FREQ");
		lblCom2Swap.setName("COM2SWAP");
		lblCom2Stby.setName("COM2STBY");
		lblNav1Freq.setName("NAV1FREQ");
		lblNav1Swap.setName("NAV1SWAP");
		lblNav1Stby.setName("NAV1STBY");
		lblNav2Freq.setName("NAV2FREQ");
		lblNav2Swap.setName("NAV2SWAP");
		lblNav2Stby.setName("NAV2STBY");
		lblAdfFreq1.setName("ADF1FREQ");
		lblXpdrCode.setName("XPDRCODE");

		lblCom1Freq.addMouseListener( owner );
		lblCom1Swap.addMouseListener( owner );
		lblCom1Stby.addMouseListener( owner );
		lblCom2Freq.addMouseListener( owner );
		lblCom2Swap.addMouseListener( owner );
		lblCom2Stby.addMouseListener( owner );
		lblNav1Freq.addMouseListener( owner );
		lblNav1Swap.addMouseListener( owner );
		lblNav1Stby.addMouseListener( owner );
		lblNav2Freq.addMouseListener( owner );
		lblNav2Swap.addMouseListener( owner );
		lblNav2Stby.addMouseListener( owner );
		lblAdfFreq1.addMouseListener( owner );
		lblXpdrCode.addMouseListener( owner );

		lblCom1Stby.addMouseWheelListener( owner );
		lblCom2Stby.addMouseWheelListener( owner );
		lblNav1Stby.addMouseWheelListener( owner );
		lblNav2Stby.addMouseWheelListener( owner );
		lblAdfFreq1.addMouseWheelListener( owner );
		lblXpdrCode.addMouseWheelListener( owner );

		add(pnlCom1);
		add(pnlCom2);
		add(pnlNav1);
		add(pnlNav2);
		add(pnlDME);
		add(pnlAdf1);
		//if( owner.debug ) add( pnlFPS );
		
		freqNF.setMaximumFractionDigits(2);
		freqNF.setMinimumFractionDigits(2);

		clkNF.setMaximumFractionDigits(0);
		clkNF.setMinimumFractionDigits(0);
		clkNF.setMaximumIntegerDigits(2);
		clkNF.setMinimumIntegerDigits(2);

		dmeNF.setMaximumFractionDigits(2);
		dmeNF.setMinimumFractionDigits(2);
		dmeNF.setMaximumIntegerDigits(2);
		dmeNF.setMinimumIntegerDigits(2);

	    Timer timer = new Timer();
	    timer.scheduleAtFixedRate(new TimerTask() {
	            public void run() {
	            	if( System.currentTimeMillis() - lasttime > 1000 ) {
	            		lasttime = System.currentTimeMillis();
		            	// System.out.println( (new Date()).toString() +" GBFPS: "+ ((Variometer)owner.gauges.get(6)).getFrames());
		                setGBFPS(frames);
		                frames = 0;
	            	}
	            }
	        }, 1000, 1000);

	}
	
	public void update() {	
		if(varCom1Xmit==null)return;
		frames ++;

		aviMaster = true; // (Double)varAviMaster.getValue() == -1;

		if( !aviMaster ) {
			lblCom1Freq.setText("     ");
			lblCom2Freq.setText("     ");
			lblCom1Stby.setText("     ");
			lblCom2Stby.setText("     ");
			lblNav1Freq.setText("     ");
			lblNav2Freq.setText("     ");
			lblNav1Stby.setText("     ");
			lblNav2Stby.setText("     ");
			lblAdfFreq1.setText("     ");
			lblXpdrCode.setText("     ");
			lblZuluTime.setText("     ");
			lblDME1.setText("     ");
			lblDME2.setText("     ");
			return;
		}
		
		com1Xmit = varCom1Xmit.getDoubleValue();
		com2Xmit = varCom2Xmit.getDoubleValue();
		nav1Xmit = varNav1Xmit.getDoubleValue();
		nav2Xmit = varNav2Xmit.getDoubleValue();
		adf1Xmit = varAdf1Xmit.getDoubleValue();
		comBoth  = varComBoth.getDoubleValue();
		com1Freq = varCom1Freq.getDoubleValue();
		com2Freq = varCom2Freq.getDoubleValue();
		com1Stby = varCom1Stby.getDoubleValue();
		com2Stby = varCom2Stby.getDoubleValue();
		nav1Freq = varNav1Freq.getDoubleValue();
		nav2Freq = varNav2Freq.getDoubleValue();
		nav1Stby = varNav1Stby.getDoubleValue();
		nav2Stby = varNav2Stby.getDoubleValue();
		adfFreq  = varAdfFreq.getDoubleValue();
		xpdrCode = varXpdrCode.getDoubleValue();
		zuluTime = varZuluTime.getDoubleValue();

		tbCom1.setTitle("COM1 ["+ ( com1Xmit == 1.0 ? "X" : "_" ) +"]"); 
		tbCom2.setTitle("COM2 ["+ ( com2Xmit == 1.0 ? "X" : "_" ) +"]"); 
		tbNav1.setTitle("NAV1 ["+ ( nav1Xmit == 1.0 ? "X" : "_" ) +"]"); 
		tbNav2.setTitle("NAV2 ["+ ( nav2Xmit == 1.0 ? "X" : "_" ) +"]"); 
		tbAdf1.setTitle("ADF ["+ ( adf1Xmit == 1.0 ? "X" : "_" ) +"] / XPDR");
		tbDME.setTitle("DME ["+ ( dmeXmit.getBooleanValue() ? "X" : "_" ) 
					+ "] MRKR ["+ ( markerXmit.getBooleanValue() ? "X" : "_" ) +"]");
		
		pnlCom1.setBorder( BorderFactory.createCompoundBorder( tbCom1, BorderFactory.createEmptyBorder(0,4,0,4))); 
		pnlCom2.setBorder( BorderFactory.createCompoundBorder( tbCom2, BorderFactory.createEmptyBorder(0,4,0,4))); 
		pnlNav1.setBorder( BorderFactory.createCompoundBorder( tbNav1, BorderFactory.createEmptyBorder(0,4,0,4))); 
		pnlNav2.setBorder( BorderFactory.createCompoundBorder( tbNav2, BorderFactory.createEmptyBorder(0,4,0,4))); 
		pnlAdf1.setBorder( BorderFactory.createCompoundBorder( tbAdf1, BorderFactory.createEmptyBorder(0,4,0,4))); 
		
		lblCom1Freq.setText("1"+(freqNF.format((float)Integer.parseInt(Integer.toHexString((int)com1Freq)) / 100)));
		lblCom2Freq.setText("1"+(freqNF.format((float)Integer.parseInt(Integer.toHexString((int)com2Freq)) / 100)));
		lblCom1Stby.setText("1"+(freqNF.format((float)Integer.parseInt(Integer.toHexString((int)com1Stby)) / 100)));
		lblCom2Stby.setText("1"+(freqNF.format((float)Integer.parseInt(Integer.toHexString((int)com2Stby)) / 100)));
		lblNav1Freq.setText(""+freqNF.format(nav1Freq));
		lblNav2Freq.setText(""+freqNF.format(nav2Freq));
		lblNav1Stby.setText(""+freqNF.format(nav1Stby));
		lblNav2Stby.setText(""+freqNF.format(nav2Stby));
		lblAdfFreq1.setText(""+lz((float)Integer.parseInt(Integer.toHexString((int)adfFreq)) / 10000, 6 ));
		lblXpdrCode.setText(""+lz(Integer.parseInt(Integer.toHexString((int)xpdrCode)), 4 ));
		lblZuluTime.setText(""+clkNF.format((int)zuluTime/3600)+":"+clkNF.format((int)zuluTime/60%60)+":"+clkNF.format((int)zuluTime%3600)+"Z");

		lblDME1.setText(""+ ((Double)hasDME1.getValue() == -1.0 ? dmeNF.format((Double)navDME1.getValue()) : "--,--"));
		lblDME2.setText(""+ ((Double)hasDME2.getValue() == -1.0 ? dmeNF.format((Double)navDME2.getValue()) : "--,--"));
		//System.out.println("ADF Freq: RAW: "+ adfFreq +" HEX: "+ Integer.toHexString((int)adfFreq));

		if( adf1Xmit == 1.0 ) {
			int freq = Integer.parseInt(Integer.toHexString((int)adfFreq)) / 10000;
			if( freq != radioPlaying ) {
				switch( freq ) {
				case 756:
				    player.selectSource("http://dradio.ic.llnwd.net/stream/dradio_dlf_m_a.ogg");
				    if( radioPlaying != 756 ) player.play_sound();
				    radioPlaying = 756;
				    break;
				case 1296:
				    player.selectSource("http://c13010-ls.i.core.cdn.streamfarm.net/dwworldlive-live/13010dwren32.ogg");
				    if( radioPlaying != 1296 ) player.play_sound();
				    radioPlaying = 1296;
				    break;
				default:
					player.stop_sound();
					radioPlaying = -1;
				}
			}
		}
		else {
			player.stop_sound();
			radioPlaying = -1;
		}
	}

	public void repaint() {
		update();
		super.repaint();
	}
	
	public void stopRadio() {
		if( radioPlaying != -1 ) {
			radioPlaying = -1;
			player.stop_sound();
		}
	}

	long lasttime = 0;
	public void setFPS(float fps) {
		lblFPS.setText(""+(Math.round(fps*10))/10);
	}
	public void setGBFPS(float fps) {
		lblGBFPS.setText(""+(Math.round(fps*10))/10);
		String title = gbTitle;
		if( atcType.getValue() != null ) {
			title = gbTitle +" - "+ atcType.getStringValue(); 
			title += " "+ atcModel.getStringValue();
			title += " - "+ atcID.getStringValue();
			title += " - FPS FSX: "+lblFPS.getText();
			//title += " GB: "+(Math.round(fps));
			title += " - www.gaugebook.org";
		}
		owner.setTitle( title );
	}
}
