package org.gaugebook.swing;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;


/**
 * The class JDecimalTextField
 *
 * @author Michael Nagler
 * @version 1.0
 */

public class JDecimalTextField extends JTextField {

    public JDecimalTextField( int cols ) {
        super( cols );
        setHorizontalAlignment( super.RIGHT );
        setDocument( new DecimalDocument( cols ) );
    }

    static class DecimalDocument extends PlainDocument {
    	
    	private int maxlen;
    	
    	public DecimalDocument( int maxlen ) {
    		this.maxlen = maxlen;
    	}
    	
        public void insertString(int offs, String str, AttributeSet a)
            throws BadLocationException {

            if (str == null) {
                return;
            }

            char[] nums   = str.toCharArray();
            char[] result = new char[ nums.length ];
            int count = 0;
            for( int i = 0; i < nums.length && i < maxlen; i++ ) {
                if( Character.isDigit( nums[i] )) {
                    result[ count++ ] = nums[i];
                }
            }
            super.insertString( offs, new String( nums, 0, count ), a );
        }
    }
}