/* JTransponderTextField.java
 */

package org.gaugebook.swing;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;


/**
 * JTransponderTextField
 *
 * @author Michael Nagler
 * @version 1.0
 */

public class JTransponderTextField extends JTextField {

    public JTransponderTextField( int cols ) {
        super( cols );
        setHorizontalAlignment( super.RIGHT );
        setDocument( new TransponderDocument() );
    }

    static class TransponderDocument extends PlainDocument {

        public void insertString(int offs, String str, AttributeSet a)
            throws BadLocationException {

            if (str == null) {
                return;
            }

            char[] nums   = str.toCharArray();
            char[] result = new char[ nums.length ];
            int count = 0;
            for( int i = 0; i < nums.length && getLength() + 1 <= 4; i++ ) {
                if( Character.isDigit( nums[i] ) && nums[i] != '8' && nums[i] != '9' ) {
                    result[ count++ ] = nums[i];
                }
            }
            super.insertString( offs, new String( nums, 0, count ), a );
        }
    }
}